import core._ui_base as gui
__name__ = 'clustalO_ui'

gui.service_cmd = 'python3 ../embl_client/clustalo.py' 
gui.app_label.value = 'EMBL-ToolsClustalO webservice'

# Define more widgets and add them to guimandatory_options  or gui.optional_options
# Add @gui.output.capture() to methods you want to have their output printed.

# Widgets for mandatory information
sequence_type = gui.widgets.Dropdown(
    options = [('DNA','dna'), ('RNA', 'rna'),('Protein', 'protein')],
    value='dna',
    description='Sequence type:',
    style = gui.style
)

def run_checks():
    if not gui.check_email():
        return False
    if not gui.check_file():
        return False
    return True
gui.run_checks = run_checks


def prepare_command():
    command = gui.service_cmd + ' --email '+ gui.email_input.value + ' --stype ' +sequence_type.value + ' --sequence ' + gui.seq_file_input.selected  
    command += ' --asyncjob'
    return command
gui.prepare_command = prepare_command

mandatory_options =[]
for widget in gui.mandatory_options.children:
    mandatory_options.append(widget)
mandatory_options.insert(2, sequence_type)
gui.mandatory_options.children = (mandatory_options)

display(gui.app_layout)
display(gui.output)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
#
# Copyright 2012-2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Python Client Automatically generated with:
# https://github.com/ebi-wp/webservice-clients-generator
#
# EMBOSS dotmatcher (REST) web service Python client using xmltramp2.
#
# For further information see:
# https://www.ebi.ac.uk/Tools/webservices/
#
###############################################################################

from __future__ import print_function

import os
import sys
import time
import requests
import platform
from xmltramp2 import xmltramp
from optparse import OptionParser

try:
    from urllib.parse import urlparse, urlencode
    from urllib.request import urlopen, Request
    from urllib.error import HTTPError
    from urllib.request import __version__ as urllib_version
except ImportError:
    from urlparse import urlparse
    from urllib import urlencode
    from urllib2 import urlopen, Request, HTTPError
    from urllib2 import __version__ as urllib_version

# allow unicode(str) to be used in python 3
try:
    unicode('')
except NameError:
    unicode = str

# Base URL for service
baseUrl = u'https://www.ebi.ac.uk/Tools/services/rest/emboss_dotmatcher'
version = u'2019-07-03 12:51'

# Set interval for checking status
pollFreq = 3
# Output level
outputLevel = 1
# Debug level
debugLevel = 0
# Number of option arguments.
numOpts = len(sys.argv)

# Process command-line options
parser = OptionParser(add_help_option=False)

# Tool specific options (Try to print all the commands automatically)
parser.add_option('--matrix', type=str, help=('Default substitution scoring matrices.'))
parser.add_option('--stype', type=str, help=('Defines the type of the sequences to be aligned'))
parser.add_option('--asequence', type=str, help=('A free text (raw) list of sequences is simply a block of characters'
                  'representing several DNA/RNA or Protein sequences. A sequence can be'
                  'in GCG, FASTA, EMBL (Nucleotide only), GenBank, PIR, NBRF, PHYLIP or'
                  'UniProtKB/Swiss-Prot (Protein only) format. Partially formatted'
                  'sequences are not accepted. Adding a return to the end of the sequence'
                  'may help certain applications understand the input. Note that directly'
                  'using data from word processors may yield unpredictable results as'
                  'hidden/control characters may be present.'))
parser.add_option('--bsequence', type=str, help=('A free text (raw) list of sequences is simply a block of characters'
                  'representing several DNA/RNA or Protein sequences. A sequence can be'
                  'in GCG, FASTA, EMBL (Nucleotide only), GenBank, PIR, NBRF, PHYLIP or'
                  'UniProtKB/Swiss-Prot (Protein only) format. Partially formatted'
                  'sequences are not accepted. Adding a return to the end of the sequence'
                  'may help certain applications understand the input. Note that directly'
                  'using data from word processors may yield unpredictable results as'
                  'hidden/control characters may be present.'))
parser.add_option('--windowsize', type=int, help=('Window size over which to test threshold'))
parser.add_option('--threshold', type=int, help=('Threshold'))
# General options
parser.add_option('-h', '--help', action='store_true', help='Show this help message and exit.')
parser.add_option('--email', help='E-mail address.')
parser.add_option('--title', help='Job title.')
parser.add_option('--outfile', help='File name for results.')
parser.add_option('--outformat', help='Output format for results.')
parser.add_option('--asyncjob', action='store_true', help='Asynchronous mode.')
parser.add_option('--jobid', help='Job identifier.')
parser.add_option('--polljob', action="store_true", help='Get job result.')
parser.add_option('--pollFreq', type='int', default=3, help='Poll frequency in seconds (default 3s).')
parser.add_option('--status', action="store_true", help='Get job status.')
parser.add_option('--resultTypes', action='store_true', help='Get result types.')
parser.add_option('--params', action='store_true', help='List input parameters.')
parser.add_option('--paramDetail', help='Get details for parameter.')
parser.add_option('--quiet', action='store_true', help='Decrease output level.')
parser.add_option('--verbose', action='store_true', help='Increase output level.')
parser.add_option('--version', action='store_true', help='Prints out the version of the Client and exit.')
parser.add_option('--debugLevel', type='int', default=debugLevel, help='Debugging level.')
parser.add_option('--baseUrl', default=baseUrl, help='Base URL for service.')

(options, args) = parser.parse_args()

# Increase output level
if options.verbose:
    outputLevel += 1

# Decrease output level
if options.quiet:
    outputLevel -= 1

# Debug level
if options.debugLevel:
    debugLevel = options.debugLevel

if options.pollFreq:
    pollFreq = options.pollFreq

if options.baseUrl:
    baseUrl = options.baseUrl


# Debug print
def printDebugMessage(functionName, message, level):
    if (level <= debugLevel):
        print(u'[' + functionName + u'] ' + message, file=sys.stderr)


# User-agent for request (see RFC2616).
def getUserAgent():
    printDebugMessage(u'getUserAgent', u'Begin', 11)
    # Agent string for urllib2 library.
    urllib_agent = u'Python-urllib/%s' % urllib_version
    clientRevision = version
    # Prepend client specific agent string.
    try:
        pythonversion = platform.python_version()
        pythonsys = platform.system()
    except ValueError:
        pythonversion, pythonsys = "Unknown", "Unknown"
    user_agent = u'EBI-Sample-Client/%s (%s; Python %s; %s) %s' % (
        clientRevision, os.path.basename(__file__),
        pythonversion, pythonsys, urllib_agent)
    printDebugMessage(u'getUserAgent', u'user_agent: ' + user_agent, 12)
    printDebugMessage(u'getUserAgent', u'End', 11)
    return user_agent


# Wrapper for a REST (HTTP GET) request
def restRequest(url):
    printDebugMessage(u'restRequest', u'Begin', 11)
    printDebugMessage(u'restRequest', u'url: ' + url, 11)
    try:
        # Set the User-agent.
        user_agent = getUserAgent()
        http_headers = {u'User-Agent': user_agent}
        req = Request(url, None, http_headers)
        # Make the request (HTTP GET).
        reqH = urlopen(req)
        resp = reqH.read()
        contenttype = reqH.info()

        if (len(resp) > 0 and contenttype != u"image/png;charset=UTF-8"
                and contenttype != u"image/jpeg;charset=UTF-8"
                and contenttype != u"application/gzip;charset=UTF-8"):
            try:
                result = unicode(resp, u'utf-8')
            except UnicodeDecodeError:
                result = resp
        else:
            result = resp
        reqH.close()
    # Errors are indicated by HTTP status codes.
    except HTTPError as ex:
        result = requests.get(url).content
    printDebugMessage(u'restRequest', u'End', 11)
    return result


# Get input parameters list
def serviceGetParameters():
    printDebugMessage(u'serviceGetParameters', u'Begin', 1)
    requestUrl = baseUrl + u'/parameters'
    printDebugMessage(u'serviceGetParameters', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetParameters', u'End', 1)
    return doc[u'id':]


# Print list of parameters
def printGetParameters():
    printDebugMessage(u'printGetParameters', u'Begin', 1)
    idList = serviceGetParameters()
    for id_ in idList:
        print(id_)
    printDebugMessage(u'printGetParameters', u'End', 1)


# Get input parameter information
def serviceGetParameterDetails(paramName):
    printDebugMessage(u'serviceGetParameterDetails', u'Begin', 1)
    printDebugMessage(u'serviceGetParameterDetails', u'paramName: ' + paramName, 2)
    requestUrl = baseUrl + u'/parameterdetails/' + paramName
    printDebugMessage(u'serviceGetParameterDetails', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetParameterDetails', u'End', 1)
    return doc


# Print description of a parameter
def printGetParameterDetails(paramName):
    printDebugMessage(u'printGetParameterDetails', u'Begin', 1)
    doc = serviceGetParameterDetails(paramName)
    print(unicode(doc.name) + u"\t" + unicode(doc.type))
    print(doc.description)
    if hasattr(doc, 'values'):
        for value in doc.values:
            print(value.value)
            if unicode(value.defaultValue) == u'true':
                print(u'default')
            print(u"\t" + unicode(value.label))
            if hasattr(value, u'properties'):
                for wsProperty in value.properties:
                    print(u"\t" + unicode(wsProperty.key) + u"\t" + unicode(wsProperty.value))
    printDebugMessage(u'printGetParameterDetails', u'End', 1)


# Submit job
def serviceRun(email, title, params):
    printDebugMessage(u'serviceRun', u'Begin', 1)
    # Insert e-mail and title into params
    params[u'email'] = email
    if title:
        params[u'title'] = title
    requestUrl = baseUrl + u'/run/'
    printDebugMessage(u'serviceRun', u'requestUrl: ' + requestUrl, 2)

    # Get the data for the other options
    requestData = urlencode(params)

    printDebugMessage(u'serviceRun', u'requestData: ' + requestData, 2)
    # Errors are indicated by HTTP status codes.
    try:
        # Set the HTTP User-agent.
        user_agent = getUserAgent()
        http_headers = {u'User-Agent': user_agent}
        req = Request(requestUrl, None, http_headers)
        # Make the submission (HTTP POST).
        reqH = urlopen(req, requestData.encode(encoding=u'utf_8', errors=u'strict'))
        jobId = unicode(reqH.read(), u'utf-8')
        reqH.close()
    except HTTPError as ex:
        print(xmltramp.parse(unicode(ex.read(), u'utf-8'))[0][0])
        quit()
    printDebugMessage(u'serviceRun', u'jobId: ' + jobId, 2)
    printDebugMessage(u'serviceRun', u'End', 1)
    return jobId


# Get job status
def serviceGetStatus(jobId):
    printDebugMessage(u'serviceGetStatus', u'Begin', 1)
    printDebugMessage(u'serviceGetStatus', u'jobId: ' + jobId, 2)
    requestUrl = baseUrl + u'/status/' + jobId
    printDebugMessage(u'serviceGetStatus', u'requestUrl: ' + requestUrl, 2)
    status = restRequest(requestUrl)
    printDebugMessage(u'serviceGetStatus', u'status: ' + status, 2)
    printDebugMessage(u'serviceGetStatus', u'End', 1)
    return status


# Print the status of a job
def printGetStatus(jobId):
    printDebugMessage(u'printGetStatus', u'Begin', 1)
    status = serviceGetStatus(jobId)
    if outputLevel > 0:
        print("Getting status for job %s" % jobId)
    print(status)
    if outputLevel > 0 and status == "FINISHED":
        print("To get results: python %s --polljob --jobid %s"
              "" % (os.path.basename(__file__), jobId))
    printDebugMessage(u'printGetStatus', u'End', 1)


# Get available result types for job
def serviceGetResultTypes(jobId):
    printDebugMessage(u'serviceGetResultTypes', u'Begin', 1)
    printDebugMessage(u'serviceGetResultTypes', u'jobId: ' + jobId, 2)
    requestUrl = baseUrl + u'/resulttypes/' + jobId
    printDebugMessage(u'serviceGetResultTypes', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetResultTypes', u'End', 1)
    return doc[u'type':]


# Print list of available result types for a job.
def printGetResultTypes(jobId):
    printDebugMessage(u'printGetResultTypes', u'Begin', 1)
    if outputLevel > 0:
        print("Getting result types for job %s" % jobId)

    resultTypeList = serviceGetResultTypes(jobId)
    if outputLevel > 0:
        print("Available result types:")
    for resultType in resultTypeList:
        print(resultType[u'identifier'])
        if hasattr(resultType, u'label'):
            print(u"\t", resultType[u'label'])
        if hasattr(resultType, u'description'):
            print(u"\t", resultType[u'description'])
        if hasattr(resultType, u'mediaType'):
            print(u"\t", resultType[u'mediaType'])
        if hasattr(resultType, u'fileSuffix'):
            print(u"\t", resultType[u'fileSuffix'])
    if outputLevel > 0:
        print("To get results:\n  python %s --polljob --jobid %s\n"
              "  python %s --polljob --outformat <type> --jobid %s"
              "" % (os.path.basename(__file__), jobId,
                    os.path.basename(__file__), jobId))
    printDebugMessage(u'printGetResultTypes', u'End', 1)


# Get result
def serviceGetResult(jobId, type_):
    printDebugMessage(u'serviceGetResult', u'Begin', 1)
    printDebugMessage(u'serviceGetResult', u'jobId: ' + jobId, 2)
    printDebugMessage(u'serviceGetResult', u'type_: ' + type_, 2)
    requestUrl = baseUrl + u'/result/' + jobId + u'/' + type_
    result = restRequest(requestUrl)
    printDebugMessage(u'serviceGetResult', u'End', 1)
    return result


# Client-side poll
def clientPoll(jobId):
    printDebugMessage(u'clientPoll', u'Begin', 1)
    result = u'PENDING'
    while result == u'RUNNING' or result == u'PENDING':
        result = serviceGetStatus(jobId)
        if outputLevel > 0:
            print(result)
        if result == u'RUNNING' or result == u'PENDING':
            time.sleep(pollFreq)
    printDebugMessage(u'clientPoll', u'End', 1)


# Get result for a jobid
# Allows more than one output file written when 'outformat' is defined.
def getResult(jobId):
    printDebugMessage(u'getResult', u'Begin', 1)
    printDebugMessage(u'getResult', u'jobId: ' + jobId, 1)
    if outputLevel > 1:
        print("Getting results for job %s" % jobId)
    # Check status and wait if necessary
    clientPoll(jobId)
    # Get available result types
    resultTypes = serviceGetResultTypes(jobId)

    for resultType in resultTypes:
        # Derive the filename for the result
        if options.outfile:
            filename = (options.outfile + u'.' + unicode(resultType[u'identifier']) +
                        u'.' + unicode(resultType[u'fileSuffix']))
        else:
            filename = (jobId + u'.' + unicode(resultType[u'identifier']) +
                        u'.' + unicode(resultType[u'fileSuffix']))
        # Write a result file

        outformat_parm = str(options.outformat).split(',')
        for outformat_type in outformat_parm:
            outformat_type = outformat_type.replace(' ', '')

            if outformat_type == 'None':
                outformat_type = None

            if not outformat_type or outformat_type == unicode(resultType[u'identifier']):
                if outputLevel > 1:
                    print("Getting %s" % unicode(resultType[u'identifier']))
                # Get the result
                result = serviceGetResult(jobId, unicode(resultType[u'identifier']))
                if (unicode(resultType[u'mediaType']) == u"image/png"
                        or unicode(resultType[u'mediaType']) == u"image/jpeg"
                        or unicode(resultType[u'mediaType']) == u"application/gzip"):
                    fmode = 'wb'
                else:
                    fmode = 'w'

                try:
                    fh = open(filename, fmode)
                    fh.write(result)
                    fh.close()
                except TypeError:
                    fh.close()
                    fh = open(filename, "wb")
                    fh.write(result)
                    fh.close()
                if outputLevel > 0:
                    print("Creating result file: " + filename)
    printDebugMessage(u'getResult', u'End', 1)


# Read a file
def readFile(filename):
    printDebugMessage(u'readFile', u'Begin', 1)
    fh = open(filename, 'r')
    data = fh.read()
    fh.close()
    printDebugMessage(u'readFile', u'End', 1)
    return data


def print_usage():
    print("""\
EMBL-EBI EMBOSS dotmatcher Python Client:

Sequence statistics and plots with dotmatcher.

[Required (for job submission)]
  --email               E-mail address.
  --stype               Defines the type of the sequences to be aligned.
  --asequence           A free text (raw) list of sequences is simply a block of
                        characters representing several DNA/RNA or Protein
                        sequences. A sequence can be in GCG, FASTA, EMBL (Nucleotide
                        only), GenBank, PIR, NBRF, PHYLIP or UniProtKB/Swiss-Prot
                        (Protein only) format. Partially formatted sequences are not
                        accepted. Adding a return to the end of the sequence may
                        help certain applications understand the input. Note that
                        directly using data from word processors may yield
                        unpredictable results as hidden/control characters may be
                        present.
  --bsequence           A free text (raw) list of sequences is simply a block of
                        characters representing several DNA/RNA or Protein
                        sequences. A sequence can be in GCG, FASTA, EMBL (Nucleotide
                        only), GenBank, PIR, NBRF, PHYLIP or UniProtKB/Swiss-Prot
                        (Protein only) format. Partially formatted sequences are not
                        accepted. Adding a return to the end of the sequence may
                        help certain applications understand the input. Note that
                        directly using data from word processors may yield
                        unpredictable results as hidden/control characters may be
                        present.

[Optional]
  --matrix              Default substitution scoring matrices.
  --windowsize          Window size over which to test threshold.
  --threshold           Threshold.

[General]
  -h, --help            Show this help message and exit.
  --asyncjob            Forces to make an asynchronous query.
  --title               Title for job.
  --status              Get job status.
  --resultTypes         Get available result types for job.
  --polljob             Poll for the status of a job.
  --pollFreq            Poll frequency in seconds (default 3s).
  --jobid               JobId that was returned when an asynchronous job was submitted.
  --outfile             File name for results (default is JobId; for STDOUT).
  --outformat           Result format(s) to retrieve. It accepts comma-separated values.
  --params              List input parameters.
  --paramDetail         Display details for input parameter.
  --verbose             Increase output.
  --version             Prints out the version of the Client and exit.
  --quiet               Decrease output.
  --baseUrl             Base URL. Defaults to:
                        https://www.ebi.ac.uk/Tools/services/rest/emboss_dotmatcher

Synchronous job:
  The results/errors are returned as soon as the job is finished.
  Usage: python emboss_dotmatcher.py --email <your@email.com> [options...] <SeqFile|SeqID(s)>
  Returns: results as an attachment

Asynchronous job:
  Use this if you want to retrieve the results at a later time. The results
  are stored for up to 24 hours.
  Usage: python emboss_dotmatcher.py --asyncjob --email <your@email.com> [options...] <SeqFile|SeqID(s)>
  Returns: jobid

Check status of Asynchronous job:
  Usage: python emboss_dotmatcher.py --status --jobid <jobId>

Retrieve job data:
  Use the jobid to query for the status of the job. If the job is finished,
  it also returns the results/errors.
  Usage: python emboss_dotmatcher.py --polljob --jobid <jobId> [--outfile string]
  Returns: string indicating the status of the job and if applicable, results
  as an attachment.

Further information:
  https://www.ebi.ac.uk/Tools/webservices and
    https://github.com/ebi-wp/webservice-clients

Support/Feedback:
  https://www.ebi.ac.uk/support/""")


# No options... print help.
if numOpts < 2:
    print_usage()
elif options.help:
    print_usage()
# List parameters
elif options.params:
    printGetParameters()
# Get parameter details
elif options.paramDetail:
    printGetParameterDetails(options.paramDetail)
# Print Client version
elif options.version:
    print("Revision: %s" % version)
    sys.exit()
# Submit job
elif options.email and not options.jobid:
    params = {}
    if len(args) == 1 and "true" not in args and "false" not in args:
        if os.path.exists(args[0]):  # Read file into content
            params[u'sequence'] = readFile(args[0])
        else:  # Argument is a sequence id
            params[u'sequence'] = args[0]
    elif len(args) == 2 and "true" not in args and "false" not in args:
        if os.path.exists(args[0]) and os.path.exists(args[1]):  # Read file into content
            params[u'asequence'] = readFile(args[0])
            params[u'bsequence'] = readFile(args[1])
        else:  # Argument is a sequence id
            params[u'asequence'] = args[0]
            params[u'bsequence'] = args[0]
    elif hasattr(options, "sequence") or (hasattr(options, "asequence") and hasattr(options, "bsequence")):  # Specified via option
        if hasattr(options, "sequence"):
            if os.path.exists(options.sequence):  # Read file into content
                params[u'sequence'] = readFile(options.sequence)
            else:  # Argument is a sequence id
                params[u'sequence'] = options.sequence
        elif hasattr(options, "asequence") and hasattr(options, "bsequence"):
            if os.path.exists(options.asequence) and os.path.exists(options.bsequence):  # Read file into content
                params[u'asequence'] = readFile(options.asequence)
                params[u'bsequence'] = readFile(options.bsequence)
            else:  # Argument is a sequence id
                params[u'asequence'] = options.asequence
                params[u'bsequence'] = options.bsequence

    # Pass default values and fix bools (without default value)
    if options.stype:
        params['stype'] = options.stype

    if options.matrix:
        params['matrix'] = options.matrix
    

    if options.windowsize:
        params['windowsize'] = options.windowsize
    

    if options.threshold:
        params['threshold'] = options.threshold
    


    # Submit the job
    jobId = serviceRun(options.email, options.title, params)
    if options.asyncjob: # Async mode
        print(jobId)
        if outputLevel > 0:
            print("To check status: python %s --status --jobid %s"
                  "" % (os.path.basename(__file__), jobId))
    else:
        # Sync mode
        if outputLevel > 0:
            print("JobId: " + jobId, file=sys.stderr)
        else:
            print(jobId)
        time.sleep(pollFreq)
        getResult(jobId)
# Get job status
elif options.jobid and options.status:
    printGetStatus(options.jobid)

elif options.jobid and (options.resultTypes or options.polljob):
    status = serviceGetStatus(options.jobid)
    if status == 'PENDING' or status == 'RUNNING':
        print("Error: Job status is %s. "
              "To get result types the job must be finished." % status)
        quit()
    # List result types for job
    if options.resultTypes:
        printGetResultTypes(options.jobid)
    # Get results for job
    elif options.polljob:
        getResult(options.jobid)
else:
    # Checks for 'email' parameter
    if not options.email:
        print('\nParameter "--email" is missing in your command. It is required!\n')

    print(u'Error: unrecognised argument combination', file=sys.stderr)
    print_usage()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
#
# Copyright 2012-2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#  WSDbfetch (REST) using urllib
#
# For further information see:
# https://www.ebi.ac.uk/Tools/webservices/
#
###############################################################################

from __future__ import print_function

import os
import sys
import time
import json
import platform
from optparse import OptionParser

try:
    from urllib.parse import urlparse, urlencode
    from urllib.request import urlopen, Request
    from urllib.error import HTTPError
    from urllib.request import __version__ as urllib_version
except ImportError:
    from urlparse import urlparse
    from urllib import urlencode
    from urllib2 import urlopen, Request, HTTPError
    from urllib2 import __version__ as urllib_version

# Service base URL
baseUrl = 'https://www.ebi.ac.uk/Tools/dbfetch/dbfetch'
version = u'2019-01-17 15:15'

# Output level
outputLevel = 1
# Debug level
debugLevel = 0

# Usage message


def print_usage():
    print("""\
EMBL-EBI EMBOSS WSDbfetch Python Client:

Dbfetch service enables database entry retrieval given a set of entry
identifiers, and a required data format.

Usage:
  python dbfetch.py <method> [arguments...] [--baseUrl <baseUrl>]

A number of methods are available:
  getSupportedDBs       List available databases.
  getSupportedFormats   List available databases with formats.
  getSupportedStyles    List available databases with styles.
  getDbFormats          List formats for a specifed database. Requires <dbName>.
  getFormatStyles       List styles for a specified database and format.
                        Requires <dbName> and <dbFormat>.
  fetchData             Retrive an database entry. See below for details of arguments.
  fetchBatch            Retrive database entries. See below for details of arguments.

Fetching an entry: fetchData
  python dbfetch.py fetchData <dbName:id> [format [style]]

  dbName:id  database name and entry ID or accession (e.g. UNIPROT:WAP_RAT),
             use @fileName to read identifiers from a file.
  format     format to retrive (e.g. uniprot)
  style      style to retrive (e.g. raw)


Fetching entries: fetchBatch
  python dbfetch.py fetchBatch <dbName> <idList> [format [style]]

  dbName     database name (e.g. UNIPROT)
  idList     list of entry IDs or accessions (e.g. 1433T_RAT,WAP_RAT).
             Maximum of 200 IDs or accessions.
  format     format to retrive (e.g. uniprot)
  style      style to retrive (e.g. raw)

Further information:
  https://www.ebi.ac.uk/Tools/webservices and
    https://github.com/ebi-wp/webservice-clients

Support/Feedback:
  https://www.ebi.ac.uk/support/""")


other_usage = """
      %prog fetchBatch <dbName> <id1,id2,...> [formatName [styleName]] [options...]
      %prog fetchData <dbName:id> [formatName [styleName]] [options...]
      %prog getDbFormats <dbName> [options...]
      %prog getFormatStyles <dbName> <formatName> [options...]
      %prog getSupportedDBs [options...]
      %prog getSupportedFormats [options...]
      %prog getSupportedStyles [options...]"""

description = """Fetch database entries using entry identifiers. For more information on dbfetch
refer to https://www.ebi.ac.uk/Tools/dbfetch/"""

epilog = """\
Further information:
  https://www.ebi.ac.uk/Tools/webservices
Support/Feedback:
  https://www.ebi.ac.uk/support/"""


# Debug print
def printDebugMessage(functionName, message, level):
    if (level <= debugLevel):
        print(sys.stderr, '[' + functionName + '] ' + message)


# User-agent for request.
def getUserAgent():
    printDebugMessage(u'getUserAgent', u'Begin', 11)
    # Agent string for urllib2 library.
    urllib_agent = u'Python-urllib/%s' % urllib_version
    clientRevision = version
    # Prepend client specific agent string.
    try:
        pythonversion = platform.python_version()
        pythonsys = platform.system()
    except ValueError:
        pythonversion, pythonsys = "Unknown", "Unknown"
    user_agent = u'EBI-Sample-Client/%s (%s; Python %s; %s) %s' % (
        clientRevision, os.path.basename(__file__),
        pythonversion, pythonsys, urllib_agent)
    printDebugMessage(u'getUserAgent', u'user_agent: ' + user_agent, 12)
    printDebugMessage(u'getUserAgent', u'End', 11)
    return user_agent


# Wrapper for a REST (HTTP GET) request
def restRequest(url):
    printDebugMessage('restRequest', 'Begin', 11)
    printDebugMessage('restRequest', 'url: ' + url, 11)
    try:
        user_agent = getUserAgent()
        http_headers = {
            'User-Agent': user_agent,
            'Accept-Encoding': 'gzip'
        }
        req = Request(url, None, http_headers)
        resp = urlopen(req)
        encoding = resp.info().get('Content-Encoding')
        result = None
        if encoding == None or encoding == 'identity':
            result = resp.read().decode('utf-8')
        elif encoding == 'gzip':
            result = resp.read()
            printDebugMessage('restRequest', 'result: ' + result, 21)
            gz = GzipFile(
                fileobj=StringIO(result),
                mode="r"
            )
            result = gz.read()
        else:
            raise Exception('Unsupported Content-Encoding')
        resp.close()
    except HTTPError as ex:
        print(ex.read())
        raise
    printDebugMessage('restRequest', 'result: ' + result, 11)
    printDebugMessage('restRequest', 'End', 11)
    return result


# Get database details.
def getDatabaseInfoList():
    printDebugMessage('getDatabaseInfoList', 'Begin', 11)
    requestUrl = baseUrl + '/dbfetch.databases?style=json'
    jsonDoc = restRequest(requestUrl)
    printDebugMessage('getDatabaseInfoList', 'json: ' + jsonDoc, 21)
    doc = json.loads(jsonDoc)
    databaseInfoList = [doc[db] for db in doc]
    printDebugMessage('getDatabaseInfoList', 'End', 11)
    return databaseInfoList

# Get list of database names.
def getSupportedDbs():
    printDebugMessage('getSupportedDbs', 'Begin', 1)
    dbList = []
    dbInfoList = getDatabaseInfoList()
    for dbInfo in dbInfoList:
        dbList.append(str(dbInfo["name"]))
    printDebugMessage('getSupportedDbs', 'End', 1)
    return dbList

# Get list of database names + formats.
def getSupportedFormats():
    printDebugMessage('getSupportedFormats', 'Begin', 1)
    dbList = []
    dbInfoList = getDatabaseInfoList()
    for dbInfo in dbInfoList:
        dbList.append("%s\t%s" % (str(dbInfo["name"]),
                                  ",".join([f["name"] for f in dbInfo["formatInfoList"]])))
    printDebugMessage('getSupportedFormats', 'End', 1)
    return dbList

def getSupportedStyles():
    printDebugMessage('getSupportedStyles', 'Begin', 1)
    dbList = []
    dbInfoList = getDatabaseInfoList()
    for dbInfo in dbInfoList:
        for format in dbInfo["formatInfoList"]:
            dbList.append("%s\t%s\t%s" % (str(dbInfo["name"]), format["name"],
                                          ",".join([s["name"] for s in format["styleInfoList"]])))
    printDebugMessage('getSupportedStyles', 'End', 1)
    return dbList


# Get list of formats for a database.
def getDbFormats(db):
    printDebugMessage('getDbFormats', 'Begin', 1)
    printDebugMessage('getDbFormats', 'db: ' + db, 2)
    formatNameList = []
    dbInfoList = getDatabaseInfoList()
    for dbInfo in dbInfoList:
        if db == dbInfo["name"]:
            formatNameList = [f["name"] for f in dbInfo["formatInfoList"]]
    printDebugMessage('getDbFormats', 'End', 1)
    return formatNameList


# Get list of styles for a format of a database.
def getFormatStyles(db, format):
    printDebugMessage('getFormatStyles', 'Begin', 1)
    styleNameList = []
    dbInfoList = getDatabaseInfoList()
    for dbInfo in dbInfoList:
        if db == dbInfo["name"]:
            for f in dbInfo["formatInfoList"]:
                if format == f["name"]:
                    for s in f["styleInfoList"]:
                        styleNameList = [s["name"] for s in f["styleInfoList"]]
    printDebugMessage('getFormatStyles', 'End', 1)
    return styleNameList


# Get an entry.
def fetchData(query, format='default', style='raw'):
    printDebugMessage('fetchData', 'Begin', 1)
    if query.startswith('@'):
        result = []
        if os.path.exists(query.lstrip('@')):
            with open(query.lstrip('@'), 'r') as inlines:
                for line in inlines:
                    requestUrl = baseUrl + '/' + line.strip().replace(':', '/') + '/' + \
                        format + '?style=' + style
                    result.append(restRequest(requestUrl))
        else:
            print("Error: unable to open file %s (No such file or directory)" % query)
        result = "".join(result)
    else:
        requestUrl = baseUrl + '/' + \
            query.replace(':', '/') + '/' + format + '?style=' + style
        result = restRequest(requestUrl)
    printDebugMessage('fetchData', 'End', 1)
    return result


# Get a set of entries.
def fetchBatch(db, idListStr, format='default', style='raw'):
    printDebugMessage('fetchBatch', 'Begin', 1)
    requestUrl = baseUrl + '/' + db + '/' + \
        idListStr + '/' + format + '?style=' + style
    result = restRequest(requestUrl)
    printDebugMessage('fetchBatch', 'End', 1)
    return result


if __name__ == '__main__':
    # Process command-line options
    # parser = OptionParser(usage=usage, description=description, epilog=epilog, version=version)
    parser = OptionParser(add_help_option=False)
    parser.add_option('-h', '--help', action='store_true',
                      help='Shows this message and exit.')
    parser.add_option('--quiet', action='store_true',
                      help='decrease output level')
    parser.add_option('--verbose', action='store_true',
                      help='increase output level')
    parser.add_option('--version', action='store_true',
                      help='Prints out the version of the Client and exit.')
    parser.add_option('--baseUrl', default=baseUrl,
                      help='base URL for dbfetch')
    parser.add_option('--debugLevel', type='int',
                      default=debugLevel, help='debug output level')
    (options, args) = parser.parse_args()

    # Print Client version
    if options.version:
        print("Revision: %s" % version)
        sys.exit()

    # No arguments, print usage
    if len(args) < 1:
        print_usage()
        sys.exit()

    if options.help:
        print_usage()
        sys.exit()

    # Increase output level.
    if options.verbose:
        outputLevel += 1

    # Decrease output level.
    if options.quiet:
        outputLevel -= 1

    # Debug level.
    if options.debugLevel:
        debugLevel = options.debugLevel

    # Base URL for service.
    if options.baseUrl:
        baseUrl = options.baseUrl

    # List databases.
    if args[0] == 'getSupportedDBs':
        dbNameList = getSupportedDbs()
        for dbName in dbNameList:
            print(dbName)
    elif args[0] == 'getSupportedFormats':
        dbNameFormatList = getSupportedFormats()
        for dbNameFormat in dbNameFormatList:
            print(dbNameFormat)
    elif args[0] == 'getSupportedStyles':
        dbNameStyleList = getSupportedStyles()
        for dbNameStyle in dbNameStyleList:
            print(dbNameStyle)
    # List formats for a database.
    elif args[0] == 'getDbFormats':
        if len(args) > 1:
            formatNameList = getDbFormats(args[1])
            if len(formatNameList) > 0:
                for formatName in formatNameList:
                    print(formatName)
            else:
                print('Database not found')
        else:
            print('<dbName> needed. See --help for more information.')
    # List formats for a database.
    elif args[0] == 'getFormatStyles':
        if len(args) > 2:
            styleNameList = getFormatStyles(args[1], args[2])
            if len(styleNameList) > 0:
                for styleName in styleNameList:
                    print(styleName)
            else:
                print('Database and format not found')
        else:
            print('<dbName> and <dbFormat> needed. See --help for more information.')
    # Fetch an entry
    elif args[0] == 'fetchData' and len(args) > 1:
        if len(args) > 3:
            print(fetchData(args[1], args[2], args[3]))
        elif len(args) > 2:
            print(fetchData(args[1], args[2]))
        else:
            print(fetchData(args[1]))
    # Fetch a set of entries
    elif args[0] == 'fetchBatch' and len(args) > 2:
        if len(args) > 4:
            print(fetchBatch(args[1], args[2], args[3], args[4]))
        elif len(args) > 3:
            print(fetchBatch(args[1], args[2], args[3]))
        else:
            print(fetchBatch(args[1], args[2]))
    # Unknown argument combination, display usage
    else:
        print('Error: unrecognised argument combination')
        print_usage()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
#
# Copyright 2012-2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Python Client Automatically generated with:
# https://github.com/ebi-wp/webservice-clients-generator
#
# Kalign (REST) web service Python client using xmltramp2.
#
# For further information see:
# https://www.ebi.ac.uk/Tools/webservices/
#
###############################################################################

from __future__ import print_function

import os
import sys
import time
import requests
import platform
from xmltramp2 import xmltramp
from optparse import OptionParser

try:
    from urllib.parse import urlparse, urlencode
    from urllib.request import urlopen, Request
    from urllib.error import HTTPError
    from urllib.request import __version__ as urllib_version
except ImportError:
    from urlparse import urlparse
    from urllib import urlencode
    from urllib2 import urlopen, Request, HTTPError
    from urllib2 import __version__ as urllib_version

# allow unicode(str) to be used in python 3
try:
    unicode('')
except NameError:
    unicode = str

# Base URL for service
baseUrl = u'https://www.ebi.ac.uk/Tools/services/rest/kalign'
version = u'2019-07-03 12:51'

# Set interval for checking status
pollFreq = 3
# Output level
outputLevel = 1
# Debug level
debugLevel = 0
# Number of option arguments.
numOpts = len(sys.argv)

# Process command-line options
parser = OptionParser(add_help_option=False)

# Tool specific options (Try to print all the commands automatically)
parser.add_option('--stype', type=str, help=('Indicates if the sequences to align are protein or nucleotide'
                  '(DNA/RNA).'))
parser.add_option('--format', type=str, help=('Format for generated multiple sequence alignment.'))
parser.add_option('--gapopen', type=str, help=('The penalty for opening/closing a gap. Half the value will be'
                  'subtracted from the alignment score when opening, and half when'
                  'closing a gap.'))
parser.add_option('--gapext', type=str, help=('Penalty for extending a gap'))
parser.add_option('--termgap', type=str, help=('Penalty to extend gaps from the N/C terminal of protein or 5/3'
                  'terminal of nucleotide sequences'))
parser.add_option('--bonus', type=str, help=('A bonus score that is added to each pair of aligned residues'))
parser.add_option('--sequence', type=str, help=('Three or more sequences to be aligned can be entered directly into'
                  'this form. Sequences can be in GCG, FASTA, EMBL (Nucleotide only),'
                  'GenBank, PIR, NBRF, PHYLIP or UniProtKB/Swiss-Prot (Protein only)'
                  'format. Partially formatted sequences are not accepted. Adding a'
                  'return to the end of the sequence may help certain applications'
                  'understand the input. Note that directly using data from word'
                  'processors may yield unpredictable results as hidden/control'
                  'characters may be present. There is currently a sequence input limit'
                  'of 2000 sequences and 2MB of data.'))
# General options
parser.add_option('-h', '--help', action='store_true', help='Show this help message and exit.')
parser.add_option('--email', help='E-mail address.')
parser.add_option('--title', help='Job title.')
parser.add_option('--outfile', help='File name for results.')
parser.add_option('--outformat', help='Output format for results.')
parser.add_option('--asyncjob', action='store_true', help='Asynchronous mode.')
parser.add_option('--jobid', help='Job identifier.')
parser.add_option('--polljob', action="store_true", help='Get job result.')
parser.add_option('--pollFreq', type='int', default=3, help='Poll frequency in seconds (default 3s).')
parser.add_option('--status', action="store_true", help='Get job status.')
parser.add_option('--resultTypes', action='store_true', help='Get result types.')
parser.add_option('--params', action='store_true', help='List input parameters.')
parser.add_option('--paramDetail', help='Get details for parameter.')
parser.add_option('--quiet', action='store_true', help='Decrease output level.')
parser.add_option('--verbose', action='store_true', help='Increase output level.')
parser.add_option('--version', action='store_true', help='Prints out the version of the Client and exit.')
parser.add_option('--debugLevel', type='int', default=debugLevel, help='Debugging level.')
parser.add_option('--baseUrl', default=baseUrl, help='Base URL for service.')

(options, args) = parser.parse_args()

# Increase output level
if options.verbose:
    outputLevel += 1

# Decrease output level
if options.quiet:
    outputLevel -= 1

# Debug level
if options.debugLevel:
    debugLevel = options.debugLevel

if options.pollFreq:
    pollFreq = options.pollFreq

if options.baseUrl:
    baseUrl = options.baseUrl


# Debug print
def printDebugMessage(functionName, message, level):
    if (level <= debugLevel):
        print(u'[' + functionName + u'] ' + message, file=sys.stderr)


# User-agent for request (see RFC2616).
def getUserAgent():
    printDebugMessage(u'getUserAgent', u'Begin', 11)
    # Agent string for urllib2 library.
    urllib_agent = u'Python-urllib/%s' % urllib_version
    clientRevision = version
    # Prepend client specific agent string.
    try:
        pythonversion = platform.python_version()
        pythonsys = platform.system()
    except ValueError:
        pythonversion, pythonsys = "Unknown", "Unknown"
    user_agent = u'EBI-Sample-Client/%s (%s; Python %s; %s) %s' % (
        clientRevision, os.path.basename(__file__),
        pythonversion, pythonsys, urllib_agent)
    printDebugMessage(u'getUserAgent', u'user_agent: ' + user_agent, 12)
    printDebugMessage(u'getUserAgent', u'End', 11)
    return user_agent


# Wrapper for a REST (HTTP GET) request
def restRequest(url):
    printDebugMessage(u'restRequest', u'Begin', 11)
    printDebugMessage(u'restRequest', u'url: ' + url, 11)
    try:
        # Set the User-agent.
        user_agent = getUserAgent()
        http_headers = {u'User-Agent': user_agent}
        req = Request(url, None, http_headers)
        # Make the request (HTTP GET).
        reqH = urlopen(req)
        resp = reqH.read()
        contenttype = reqH.info()

        if (len(resp) > 0 and contenttype != u"image/png;charset=UTF-8"
                and contenttype != u"image/jpeg;charset=UTF-8"
                and contenttype != u"application/gzip;charset=UTF-8"):
            try:
                result = unicode(resp, u'utf-8')
            except UnicodeDecodeError:
                result = resp
        else:
            result = resp
        reqH.close()
    # Errors are indicated by HTTP status codes.
    except HTTPError as ex:
        result = requests.get(url).content
    printDebugMessage(u'restRequest', u'End', 11)
    return result


# Get input parameters list
def serviceGetParameters():
    printDebugMessage(u'serviceGetParameters', u'Begin', 1)
    requestUrl = baseUrl + u'/parameters'
    printDebugMessage(u'serviceGetParameters', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetParameters', u'End', 1)
    return doc[u'id':]


# Print list of parameters
def printGetParameters():
    printDebugMessage(u'printGetParameters', u'Begin', 1)
    idList = serviceGetParameters()
    for id_ in idList:
        print(id_)
    printDebugMessage(u'printGetParameters', u'End', 1)


# Get input parameter information
def serviceGetParameterDetails(paramName):
    printDebugMessage(u'serviceGetParameterDetails', u'Begin', 1)
    printDebugMessage(u'serviceGetParameterDetails', u'paramName: ' + paramName, 2)
    requestUrl = baseUrl + u'/parameterdetails/' + paramName
    printDebugMessage(u'serviceGetParameterDetails', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetParameterDetails', u'End', 1)
    return doc


# Print description of a parameter
def printGetParameterDetails(paramName):
    printDebugMessage(u'printGetParameterDetails', u'Begin', 1)
    doc = serviceGetParameterDetails(paramName)
    print(unicode(doc.name) + u"\t" + unicode(doc.type))
    print(doc.description)
    if hasattr(doc, 'values'):
        for value in doc.values:
            print(value.value)
            if unicode(value.defaultValue) == u'true':
                print(u'default')
            print(u"\t" + unicode(value.label))
            if hasattr(value, u'properties'):
                for wsProperty in value.properties:
                    print(u"\t" + unicode(wsProperty.key) + u"\t" + unicode(wsProperty.value))
    printDebugMessage(u'printGetParameterDetails', u'End', 1)


# Submit job
def serviceRun(email, title, params):
    printDebugMessage(u'serviceRun', u'Begin', 1)
    # Insert e-mail and title into params
    params[u'email'] = email
    if title:
        params[u'title'] = title
    requestUrl = baseUrl + u'/run/'
    printDebugMessage(u'serviceRun', u'requestUrl: ' + requestUrl, 2)

    # Get the data for the other options
    requestData = urlencode(params)

    printDebugMessage(u'serviceRun', u'requestData: ' + requestData, 2)
    # Errors are indicated by HTTP status codes.
    try:
        # Set the HTTP User-agent.
        user_agent = getUserAgent()
        http_headers = {u'User-Agent': user_agent}
        req = Request(requestUrl, None, http_headers)
        # Make the submission (HTTP POST).
        reqH = urlopen(req, requestData.encode(encoding=u'utf_8', errors=u'strict'))
        jobId = unicode(reqH.read(), u'utf-8')
        reqH.close()
    except HTTPError as ex:
        print(xmltramp.parse(unicode(ex.read(), u'utf-8'))[0][0])
        quit()
    printDebugMessage(u'serviceRun', u'jobId: ' + jobId, 2)
    printDebugMessage(u'serviceRun', u'End', 1)
    return jobId


# Get job status
def serviceGetStatus(jobId):
    printDebugMessage(u'serviceGetStatus', u'Begin', 1)
    printDebugMessage(u'serviceGetStatus', u'jobId: ' + jobId, 2)
    requestUrl = baseUrl + u'/status/' + jobId
    printDebugMessage(u'serviceGetStatus', u'requestUrl: ' + requestUrl, 2)
    status = restRequest(requestUrl)
    printDebugMessage(u'serviceGetStatus', u'status: ' + status, 2)
    printDebugMessage(u'serviceGetStatus', u'End', 1)
    return status


# Print the status of a job
def printGetStatus(jobId):
    printDebugMessage(u'printGetStatus', u'Begin', 1)
    status = serviceGetStatus(jobId)
    if outputLevel > 0:
        print("Getting status for job %s" % jobId)
    print(status)
    if outputLevel > 0 and status == "FINISHED":
        print("To get results: python %s --polljob --jobid %s"
              "" % (os.path.basename(__file__), jobId))
    printDebugMessage(u'printGetStatus', u'End', 1)


# Get available result types for job
def serviceGetResultTypes(jobId):
    printDebugMessage(u'serviceGetResultTypes', u'Begin', 1)
    printDebugMessage(u'serviceGetResultTypes', u'jobId: ' + jobId, 2)
    requestUrl = baseUrl + u'/resulttypes/' + jobId
    printDebugMessage(u'serviceGetResultTypes', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetResultTypes', u'End', 1)
    return doc[u'type':]


# Print list of available result types for a job.
def printGetResultTypes(jobId):
    printDebugMessage(u'printGetResultTypes', u'Begin', 1)
    if outputLevel > 0:
        print("Getting result types for job %s" % jobId)

    resultTypeList = serviceGetResultTypes(jobId)
    if outputLevel > 0:
        print("Available result types:")
    for resultType in resultTypeList:
        print(resultType[u'identifier'])
        if hasattr(resultType, u'label'):
            print(u"\t", resultType[u'label'])
        if hasattr(resultType, u'description'):
            print(u"\t", resultType[u'description'])
        if hasattr(resultType, u'mediaType'):
            print(u"\t", resultType[u'mediaType'])
        if hasattr(resultType, u'fileSuffix'):
            print(u"\t", resultType[u'fileSuffix'])
    if outputLevel > 0:
        print("To get results:\n  python %s --polljob --jobid %s\n"
              "  python %s --polljob --outformat <type> --jobid %s"
              "" % (os.path.basename(__file__), jobId,
                    os.path.basename(__file__), jobId))
    printDebugMessage(u'printGetResultTypes', u'End', 1)


# Get result
def serviceGetResult(jobId, type_):
    printDebugMessage(u'serviceGetResult', u'Begin', 1)
    printDebugMessage(u'serviceGetResult', u'jobId: ' + jobId, 2)
    printDebugMessage(u'serviceGetResult', u'type_: ' + type_, 2)
    requestUrl = baseUrl + u'/result/' + jobId + u'/' + type_
    result = restRequest(requestUrl)
    printDebugMessage(u'serviceGetResult', u'End', 1)
    return result


# Client-side poll
def clientPoll(jobId):
    printDebugMessage(u'clientPoll', u'Begin', 1)
    result = u'PENDING'
    while result == u'RUNNING' or result == u'PENDING':
        result = serviceGetStatus(jobId)
        if outputLevel > 0:
            print(result)
        if result == u'RUNNING' or result == u'PENDING':
            time.sleep(pollFreq)
    printDebugMessage(u'clientPoll', u'End', 1)


# Get result for a jobid
# Allows more than one output file written when 'outformat' is defined.
def getResult(jobId):
    printDebugMessage(u'getResult', u'Begin', 1)
    printDebugMessage(u'getResult', u'jobId: ' + jobId, 1)
    if outputLevel > 1:
        print("Getting results for job %s" % jobId)
    # Check status and wait if necessary
    clientPoll(jobId)
    # Get available result types
    resultTypes = serviceGetResultTypes(jobId)

    for resultType in resultTypes:
        # Derive the filename for the result
        if options.outfile:
            filename = (options.outfile + u'.' + unicode(resultType[u'identifier']) +
                        u'.' + unicode(resultType[u'fileSuffix']))
        else:
            filename = (jobId + u'.' + unicode(resultType[u'identifier']) +
                        u'.' + unicode(resultType[u'fileSuffix']))
        # Write a result file

        outformat_parm = str(options.outformat).split(',')
        for outformat_type in outformat_parm:
            outformat_type = outformat_type.replace(' ', '')

            if outformat_type == 'None':
                outformat_type = None

            if not outformat_type or outformat_type == unicode(resultType[u'identifier']):
                if outputLevel > 1:
                    print("Getting %s" % unicode(resultType[u'identifier']))
                # Get the result
                result = serviceGetResult(jobId, unicode(resultType[u'identifier']))
                if (unicode(resultType[u'mediaType']) == u"image/png"
                        or unicode(resultType[u'mediaType']) == u"image/jpeg"
                        or unicode(resultType[u'mediaType']) == u"application/gzip"):
                    fmode = 'wb'
                else:
                    fmode = 'w'

                try:
                    fh = open(filename, fmode)
                    fh.write(result)
                    fh.close()
                except TypeError:
                    fh.close()
                    fh = open(filename, "wb")
                    fh.write(result)
                    fh.close()
                if outputLevel > 0:
                    print("Creating result file: " + filename)
    printDebugMessage(u'getResult', u'End', 1)


# Read a file
def readFile(filename):
    printDebugMessage(u'readFile', u'Begin', 1)
    fh = open(filename, 'r')
    data = fh.read()
    fh.close()
    printDebugMessage(u'readFile', u'End', 1)
    return data


def print_usage():
    print("""\
EMBL-EBI Kalign Python Client:

Multiple sequence alignment with Kalign.

[Required (for job submission)]
  --email               E-mail address.
  --stype               Indicates if the sequences to align are protein or
                        nucleotide (DNA/RNA).
  --sequence            Three or more sequences to be aligned can be entered
                        directly into this form. Sequences can be in GCG, FASTA,
                        EMBL (Nucleotide only), GenBank, PIR, NBRF, PHYLIP or
                        UniProtKB/Swiss-Prot (Protein only) format. Partially
                        formatted sequences are not accepted. Adding a return to the
                        end of the sequence may help certain applications understand
                        the input. Note that directly using data from word
                        processors may yield unpredictable results as hidden/control
                        characters may be present. There is currently a sequence
                        input limit of 2000 sequences and 2MB of data.

[Optional]
  --format              Format for generated multiple sequence alignment.
  --gapopen             The penalty for opening/closing a gap. Half the value will
                        be subtracted from the alignment score when opening, and
                        half when closing a gap.
  --gapext              Penalty for extending a gap.
  --termgap             Penalty to extend gaps from the N/C terminal of protein or
                        5'/3' terminal of nucleotide sequences.
  --bonus               A bonus score that is added to each pair of aligned residues.

[General]
  -h, --help            Show this help message and exit.
  --asyncjob            Forces to make an asynchronous query.
  --title               Title for job.
  --status              Get job status.
  --resultTypes         Get available result types for job.
  --polljob             Poll for the status of a job.
  --pollFreq            Poll frequency in seconds (default 3s).
  --jobid               JobId that was returned when an asynchronous job was submitted.
  --outfile             File name for results (default is JobId; for STDOUT).
  --outformat           Result format(s) to retrieve. It accepts comma-separated values.
  --params              List input parameters.
  --paramDetail         Display details for input parameter.
  --verbose             Increase output.
  --version             Prints out the version of the Client and exit.
  --quiet               Decrease output.
  --baseUrl             Base URL. Defaults to:
                        https://www.ebi.ac.uk/Tools/services/rest/kalign

Synchronous job:
  The results/errors are returned as soon as the job is finished.
  Usage: python kalign.py --email <your@email.com> [options...] <SeqFile|SeqID(s)>
  Returns: results as an attachment

Asynchronous job:
  Use this if you want to retrieve the results at a later time. The results
  are stored for up to 24 hours.
  Usage: python kalign.py --asyncjob --email <your@email.com> [options...] <SeqFile|SeqID(s)>
  Returns: jobid

Check status of Asynchronous job:
  Usage: python kalign.py --status --jobid <jobId>

Retrieve job data:
  Use the jobid to query for the status of the job. If the job is finished,
  it also returns the results/errors.
  Usage: python kalign.py --polljob --jobid <jobId> [--outfile string]
  Returns: string indicating the status of the job and if applicable, results
  as an attachment.

Further information:
  https://www.ebi.ac.uk/Tools/webservices and
    https://github.com/ebi-wp/webservice-clients

Support/Feedback:
  https://www.ebi.ac.uk/support/""")


# No options... print help.
if numOpts < 2:
    print_usage()
elif options.help:
    print_usage()
# List parameters
elif options.params:
    printGetParameters()
# Get parameter details
elif options.paramDetail:
    printGetParameterDetails(options.paramDetail)
# Print Client version
elif options.version:
    print("Revision: %s" % version)
    sys.exit()
# Submit job
elif options.email and not options.jobid:
    params = {}
    if len(args) == 1 and "true" not in args and "false" not in args:
        if os.path.exists(args[0]):  # Read file into content
            params[u'sequence'] = readFile(args[0])
        else:  # Argument is a sequence id
            params[u'sequence'] = args[0]
    elif len(args) == 2 and "true" not in args and "false" not in args:
        if os.path.exists(args[0]) and os.path.exists(args[1]):  # Read file into content
            params[u'asequence'] = readFile(args[0])
            params[u'bsequence'] = readFile(args[1])
        else:  # Argument is a sequence id
            params[u'asequence'] = args[0]
            params[u'bsequence'] = args[0]
    elif hasattr(options, "sequence") or (hasattr(options, "asequence") and hasattr(options, "bsequence")):  # Specified via option
        if hasattr(options, "sequence"):
            if os.path.exists(options.sequence):  # Read file into content
                params[u'sequence'] = readFile(options.sequence)
            else:  # Argument is a sequence id
                params[u'sequence'] = options.sequence
        elif hasattr(options, "asequence") and hasattr(options, "bsequence"):
            if os.path.exists(options.asequence) and os.path.exists(options.bsequence):  # Read file into content
                params[u'asequence'] = readFile(options.asequence)
                params[u'bsequence'] = readFile(options.bsequence)
            else:  # Argument is a sequence id
                params[u'asequence'] = options.asequence
                params[u'bsequence'] = options.bsequence

    # Pass default values and fix bools (without default value)
    if options.stype:
        params['stype'] = options.stype

    if not options.format:
        params['format'] = 'clu'
    if options.format:
        params['format'] = options.format
    

    if options.gapopen:
        params['gapopen'] = options.gapopen
    

    if options.gapext:
        params['gapext'] = options.gapext
    

    if options.termgap:
        params['termgap'] = options.termgap
    

    if options.bonus:
        params['bonus'] = options.bonus
    


    # Submit the job
    jobId = serviceRun(options.email, options.title, params)
    if options.asyncjob: # Async mode
        print(jobId)
        if outputLevel > 0:
            print("To check status: python %s --status --jobid %s"
                  "" % (os.path.basename(__file__), jobId))
    else:
        # Sync mode
        if outputLevel > 0:
            print("JobId: " + jobId, file=sys.stderr)
        else:
            print(jobId)
        time.sleep(pollFreq)
        getResult(jobId)
# Get job status
elif options.jobid and options.status:
    printGetStatus(options.jobid)

elif options.jobid and (options.resultTypes or options.polljob):
    status = serviceGetStatus(options.jobid)
    if status == 'PENDING' or status == 'RUNNING':
        print("Error: Job status is %s. "
              "To get result types the job must be finished." % status)
        quit()
    # List result types for job
    if options.resultTypes:
        printGetResultTypes(options.jobid)
    # Get results for job
    elif options.polljob:
        getResult(options.jobid)
else:
    # Checks for 'email' parameter
    if not options.email:
        print('\nParameter "--email" is missing in your command. It is required!\n')

    print(u'Error: unrecognised argument combination', file=sys.stderr)
    print_usage()



__name__ = '_ui_base' 
import ipywidgets as widgets
from IPython.display import display
import subprocess, re, os, json
from time import sleep
from ipyfilechooser import FileChooser

service_cmd =""
style = {'description_width': 'initial'}
default_widget_layout = {'width':'90%'}
fixed_width_layout = widgets.Layout(width='50%', min_width='50%',max_width='50%')

# Checking if settings file exists and reading contents for default values to use.
settings = None
initLog = ''
if os.path.exists('/home/biodatahub/private/.embl_tools/settings.json'):
    with open('/home/biodatahub/private/.embl_tools/settings.json', 'r') as settingsFile:
        settingsData = settingsFile.read()
        settingsFile.close()
        settings = json.loads(settingsData)
    initLog += "Settings loaded.\n"
    for setting in settings:
        initLog += setting +': '+settings[setting]+'\n'
    if not os.path.exists(settings['outdir']):
        initLog += 'Default output directory does not exists. Attempting to create it.\n'
        try:
            os.makedirs(settings['outdir'])
            initLog += settings['outdir'] + ' is created.\n'
        except Exception as exception:
            print (type(exception), exception)
else:
    initLog += 'No settings file found.\n'
    initLog += os.path.dirname(__file__)+'../settings.json'


#Defining UI elements / widgets
app_label = widgets.Label(value='') # TODO replace with real application label text

# Widgets for mandatory information, starting with predefined label and email input
mandatory_label = widgets.Label(value='Mandatory options')
email_input = widgets.Text(value= '' if settings== None else settings['email'], placeholder='email address (mandatory)', description='Email (mandatory):',style = style, layout = default_widget_layout)
seq_file_input = FileChooser('./', title = 'Sequence file',style = style, disabled = False)
seq_file_input.use_dir_icons = True

# predefined submit and output
submit = widgets.Button(description='Submit',disabled=False, button_style='', tooltip='Submit job',style = style, icon='check')

clearOutput_button = widgets.Button(description='Clear output',disabled=False, button_style='', tooltip='Clear output',style = style)

output = widgets.Output(layout={'border': '1px solid black'})


# Widgets for optional options, add widgets as you need
optional_label = widgets.Label(value='Extra options')
output_file_name = widgets.Text(description = 'Save result as: ',style = style, disabled = False, layout=default_widget_layout)
output_dir = FileChooser( '.' if settings == None else settings['outdir'] , title = 'Save output to',style = style, disabled = False)
output_dir.default_path = '.'if settings == None else settings['outdir'] 
with output:
    print(initLog)

# Mostly generic logic

#Callback for the submitbutton, TODO modify as needed
@output.capture()
def submit_job(button):
    if not run_checks():
        return
    # add more checks here for early returns.
    
    command = prepare_command()
    print('Executing: ', command)
    proc = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
    (out, err) = proc.communicate()
    jobid = out.decode('UTF-8').split('\n')[0]
    print('Jobid: '+jobid)
    proc = subprocess.Popen([service_cmd + ' --status --jobid '+ jobid], stdout=subprocess.PIPE, shell=True)
    (out, err) = proc.communicate()
    status = out.decode('UTF-8').split('\n')[1]
        
    while status == 'RUNNING':
        sleep(5)
        proc = subprocess.Popen([service_cmd + ' --status --jobid '+ jobid], stdout=subprocess.PIPE, shell=True)
        (out, err) = proc.communicate()
        status = out.decode('UTF-8').split('\n')[1]
        print (status)
    
    fetch_result(jobid)

# This can stay as is, checks if email is valid
@output.capture()
def check_email():
    if "".__eq__(email_input.value):
        print ("Email address is empty. Please fill in the email address field")
        return False
    email_match = re.match(r".*@*\..*", email_input.value)
    if email_match == None:
        print ("Email address is not valid. Please use <account_name>@<privder> format. Example: mail@example.com")
        return False
    return True

@output.capture()
def check_file():
    file_path = seq_file_input.selected
    if file_path and os.path.isfile(file_path) and os.access(file_path, os.R_OK):
        return True
    else:
        print("Either the input file is missing or not readable")
        return False
def clear_output(button):
    output.clear_output()
    

# Should be overwritten by module.
def prepare_command():
    return command

# Util method to append correct outfile param
def append_outfile(cmd, jobid):
    command = cmd
    outfile_str = None;
    if output_dir.selected_path:
        outfile_str = output_dir.selected_path
    else:
        outfile_str = output_dir.default_path
        
    if output_file_name.value:
        outfile_str += '/'+ output_file_name.value
    else:
        outfile_str += '/'+ jobid
        
    return command + ' --outfile '+ outfile_str


def fetch_result(jobid):
    command = service_cmd + ' --polljob --jobid '+ jobid
    command = append_outfile(command, jobid)
    proc = subprocess.Popen([command], stdout=subprocess.PIPE, shell=True)
    (out, err) = proc.communicate()
    print (out.decode('UTF-8'))

# Module template file overwrites this thus the final module can extend the number of checks applied.
def run_checks():
    return True

submit.on_click(submit_job)
clearOutput_button.on_click(clear_output)
                             
# Define layout                             
mandatory_options = widgets.VBox([mandatory_label, email_input, seq_file_input, submit, clearOutput_button])
optional_options = widgets.VBox([optional_label, output_file_name,output_dir])
center_container = widgets.HBox([mandatory_options, optional_options])
mandatory_options.layout = fixed_width_layout
optional_options.layout = fixed_width_layout

app_layout = widgets.AppLayout(
    header= app_label,
    left=None,
    center = center_container,
    right= None,
    footer = None
)




import json

from notebook.base.handlers import APIHandler
from notebook.utils import url_path_join
import tornado
import os
import logging
import subprocess
_location = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))
_embl_tools_location = os.path.join(_location,'../embl_tools/')
_private_dir = os.path.join(os.environ['HOME'],'private/.embl_tools')
_log_file_path = os.path.join(_private_dir, 'embl-tools-jl.log')
_settings_file_location = os.path.join(_private_dir, 'settings.json')
_settings = None
try:
    if not os.path.exists(_private_dir):
        os.makedirs(_private_dir)
except Exception as ex:
    print("Failed to create directory for private data.")
    
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
logFileHandler = logging.FileHandler(_log_file_path, mode='a', encoding='utf-8', delay=False)
logFileHandler.setFormatter(formatter)
logger.addHandler(logFileHandler)
logger.propagate=False

# /startup endpoint
class Startup_handler(APIHandler):
    
    @property
    def contents_manager(self):
        '''Currently configured notebook server ContentsManager.'''
        return self.settings['contents_manager']

    @property
    def root_dir(self):
        '''Root directory to scan.'''
        return self.contents_manager.root_dir

    def scan_disk(self, path):
        try:
            for entry in os.scandir(path):
                if entry.is_dir():
                    if entry.name.lower() == 'embl-tools-jl':
                        continue
                    if entry.name.lower() == 'embl-tools' or entry.name.lower() == 'embl_tools':
                        return os.path.relpath(os.path.dirname(entry.path), self.root_dir)+'/'+entry.name 
                    subdir_scan=self.scan_disk(entry.path)
                    if subdir_scan:
                        return subdir_scan
            return None
        except PermissionError:
            logger.warning('Permission denied: '+ path)
        except OSError as error:
            logger.error('OSError code: %i at %s ', error.errno, path)

    # The following decorator should be present on all verb methods (head, get, post, 
    # patch, put, delete, options) to ensure only authorized user can request the 
    # Jupyter server
    @tornado.web.authenticated
    def get(self):
        embl_path = self.scan_disk(self.root_dir)
        old_location = None
        if embl_path == None:
            logger.info(f'EMBL-Tools not found in current Jupyter Lab root dir: {self.root_dir}. Copying built in version as temporary solution.')
            real_embl_path = os.path.join(self.root_dir,"embl_tools")
            embl_path = os.path.relpath('embl_tools', self.root_dir)
            logger.info(f'Executing: cp -R {_embl_tools_location} {real_embl_path})')
            try:
                subprocess.run(['cp', '-R', f'{_embl_tools_location}', f'{real_embl_path}'])
                logger.info('Copying done.')
            except Exception as ex:
                logger.error(f'Failed to copy EMBL-tools to current Jupyer Lab location\n{ex}')
            logger.info("Setting PYTHONPATH environmental variable.")
            os.environ['PYTHONPATH']=f'{os.path.abspath(embl_path)}{os.pathsep}$PYTHONPATH'
            
            if not os.path.exists(_settings_file_location):
                if not os.path.exists(_private_dir):
                    logger.info(f'Creating parent directory for settings file: {_settings_file_dir_}')                   
                    try:
                        os.makedirs(_private_dir)
                        logger.info(f'{_settings_file_dir_} created.') 
                    except Exception as ex:
                        logger.error(f'Failed to create parent directory for settings file at: {_settings_file_dir_}\n{ex}')
                try:
                    logger.info(f'Settings file not found at: {_settings_file_location}\nCreating empty settings file.')
                    with open(_settings_file_location, mode='w') as settingsFile:
                            settingsFile.write(json.dumps({}))
                            settingsFile.close()
                except Exception as ex:
                    logger.error(f'Failed create empty settings file.\n{ex}')
            else:
                logger.info(f'Settings found at: {_settings_file_location}')
            try:
                with open(_settings_file_location, mode='r') as settingsFile:
                    settingsData = settingsFile.read()
                    _settings = json.loads(settingsData)
                    settingsFile.close()
            except Exception as ex:
                logger.error(f'Failed to read settings file at startup.\n{ex}')
            old_location = None if 'tmploc' not in _settings else _settings['tmploc']
            _settings['tmploc'] = real_embl_path

            try:
                with open(_settings_file_location, mode='w') as settingsFile:
                    settingsFile.write(json.dumps(_settings))
                    settingsFile.close()
            except Exception as ex:
                logger.error(f'Failed write settings file at startup.\n{ex}')

            if old_location != None and old_location != real_embl_path:
                subprocess.run(['rm','-rf', f'{old_location}'])
                logger.info(f'Removed EMBL-tools from old temporary location: {old_location}')
        else:
            logger.info(f'EMBL-Tools found at {os.path.abspath(embl_path)}')
            logger.info("Setting PYTHONPATH environmental variable.")
            os.environ['PYTHONPATH']=f'{os.path.abspath(embl_path)}{os.pathsep}$PYTHONPATH'
            try:
                if not os.path.exists(_settings_file_location):
                    if not os.path.exists(_private_dir):
                        logger.info(f'Creating parent directory for settings file: {_settings_file_dir_}')                   
                        try:
                            os.makedirs(_private_dir)
                            logger.info(f'{_settings_file_dir_} created.') 
                        except Exception as ex:
                            logger.error(f'Failed to create parent directory for settings file at: {_settings_file_dir_}\n{ex}')
                    logger.info(f'Settings file not found at: {_settings_file_location}\nCreating empty settings file.')
                    with open(_settings_file_location, mode='w') as settingsFile:
                        settingsFile.write(json.dumps({}))
                        settingsFile.close()
                        
                with open(_settings_file_location, mode='r') as settingsFile:
                    settingsData = settingsFile.read()
                    _settings = json.loads(settingsData)
                    old_location = None if 'tmploc' not in _settings else _settings['tmploc']
                    settingsFile.close()
            except Exception as ex:
                logger.error(f'Failed to read settings file at startup. {ex}')

            if old_location != None and old_location != os.path.abspath(embl_path):
                subprocess.run(['rm', '-rf', f'{old_location}'])
                logger.info(f'Removed EMBL-tools from old temporary location: {old_location}')

        self.finish(json.dumps({
            'path': embl_path,
            'found': not(embl_path==None)
        }))

# /toolcheck endpoint
class ToolsChecker_handler(APIHandler):
    
    @property
    def contents_manager(self):
        '''Currently configured notebook server ContentsManager.'''
        return self.settings['contents_manager']

    @property
    def root_dir(self):
        '''Root directory to scan.'''
        return self.contents_manager.root_dir
    
    def scan_disk(self, path):
        tool_file_names = []
        try:
            for entry in os.scandir(path):
                if entry.is_file() and entry.name.split('.')[-1] == 'ipynb':
                    tool_file_names.append(entry.name)
            return tool_file_names
        except Exception as ex:
            logger.error(ex.__str__())

    @tornado.web.authenticated
    def post(self):
        data = self.get_json_body()
        emblTools_path = data['path']+'/notebooks'
        tools = self.scan_disk(os.path.join(self.root_dir,emblTools_path))
        self.finish(json.dumps({
            'tools':tools
        }))

# / settings endpoint
class settingsHandler(APIHandler):
    @tornado.web.authenticated
    def post(self):
        try:
            data = self.get_json_body()
            emailSetting = data['email']
            outdirSetting = data['outdir']
            if not os.path.exists(_settings_file_location):
                logger.info(f'Settings file not found at: {_settings_file_location}\nCreating empty settings file.')
                if not os.path.exists(_private_dir):
                    logger.info(f'Creating parent directory for settings file: {_settings_file_dir_}')                   
                    try:
                        os.makedirs(_private_dir)
                        logger.info(f'{_settings_file_dir_} created.') 
                    except Exception as ex:
                        logger.error(f'Failed to create parent directory for settings file at: {_settings_file_dir_}\n{ex}')
                
                with open(_settings_file_location, mode='w') as settingsFile:
                    settingsFile.write(json.dumps({}))
                    settingsFile.close()

            with open(_settings_file_location, mode='r') as settingsFile:
                settingsData = settingsFile.read()
                _settings = json.loads(settingsData)
                settingsFile.close()
            _settings['email'] = emailSetting
            _settings['outdir'] = outdirSetting
            with open(_settings_file_location, mode='w') as settingsFile:
                settingsFile.write(json.dumps(_settings))
                settingsFile.close()
            self.finish(json.dumps({
                'result': True
            }))
        except Exception as ex:
            logger.error(f'Failed to save settings. {ex.__str__()}')
            self.finish(json.dumps({
                'result': False,
                'reason':  ex.__str__()
            }))

# /descriptions endpoint
class ToolDescriptionsHandler(APIHandler):

    @tornado.web.authenticated
    def get(self):
        desc_file_path = os.path.join(_location, 'toolDescriptions.json')
        descriptions = '{}'
        success = False
        error_msg = ''
        try:
            with open(desc_file_path, 'r') as descriptions_file:
                descriptions = descriptions_file.read()
                descriptions_file.close()
                success = True
        except Exception as ex:
            error_msg  = ex.__str__()
            logger.error(error_msg)
        self.finish(json.dumps({
            'success': success,
            'error_msg' : error_msg,
            'descriptions': descriptions
        }))    

def setup_handlers(web_app):
    logger.info('embl-tools-jl extension starting up.')
    host_pattern = '.*$'
    extension_url = 'embl-tools-jl'
    base_url = web_app.settings['base_url']
    startup_pattern = url_path_join(base_url, extension_url, 'startup')
    toolcheck_pattern = url_path_join(base_url, extension_url, 'toolcheck')
    saveSettings_pattern = url_path_join(base_url, extension_url, 'settings' )
    getDescriptions_pattern = url_path_join(base_url, extension_url, 'descriptions')
    handlers = [(startup_pattern, Startup_handler), (toolcheck_pattern,ToolsChecker_handler ),
                (saveSettings_pattern, settingsHandler), (getDescriptions_pattern, ToolDescriptionsHandler)]
    web_app.add_handlers(host_pattern, handlers)
    logger.info('embl-tools-jl extension started.')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
#
# Copyright 2012-2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Python Client Automatically generated with:
# https://github.com/ebi-wp/webservice-clients-generator
#
# PRATT (REST) web service Python client using xmltramp2.
#
# For further information see:
# https://www.ebi.ac.uk/Tools/webservices/
#
###############################################################################

from __future__ import print_function

import os
import sys
import time
import requests
import platform
from xmltramp2 import xmltramp
from optparse import OptionParser

try:
    from urllib.parse import urlparse, urlencode
    from urllib.request import urlopen, Request
    from urllib.error import HTTPError
    from urllib.request import __version__ as urllib_version
except ImportError:
    from urlparse import urlparse
    from urllib import urlencode
    from urllib2 import urlopen, Request, HTTPError
    from urllib2 import __version__ as urllib_version

# allow unicode(str) to be used in python 3
try:
    unicode('')
except NameError:
    unicode = str

# Base URL for service
baseUrl = u'https://www.ebi.ac.uk/Tools/services/rest/pratt'
version = u'2019-07-03 12:51'

# Set interval for checking status
pollFreq = 3
# Output level
outputLevel = 1
# Debug level
debugLevel = 0
# Number of option arguments.
numOpts = len(sys.argv)

# Process command-line options
parser = OptionParser(add_help_option=False)

# Tool specific options (Try to print all the commands automatically)
parser.add_option('--minPerc', type=int, help=('Set the minimum percentage of the input sequences that should match a'
                  'pattern (C%). If you set this to, say 80, Pratt will only report'
                  'patterns matching at least 80 % of the sequences input.'))
parser.add_option('--patternPosition', type=str, help=('Pattern position in sequence (PP parameter)'))
parser.add_option('--maxPatternLength', type=int, help=('Maximum pattern length (PL parameter) allows you to set the maximum'
                  'length of a pattern. The length of the pattern C-x(2,4)-[DE] is'
                  '1+4+1=6. The memory requirement of Pratt depends on L; a higher L'
                  'value gives higher memory requirement.'))
parser.add_option('--maxNumPatternSymbols', type=int, help=('Maximum number of pattern symbols (PN parameter). Using this you can'
                  'set the maximum number of symbols in a pattern. The pattern'
                  'C-x(2,4)-[DE] has 2 symbols (C and [DE]). When PN is increased, Pratt'
                  'will require more memory.'))
parser.add_option('--maxNumWildcard', type=int, help=('Maximum length of a widecard (x). Using this option you can set the'
                  'maximum length of a wildcard (PX parameter). Increasing this will'
                  'increase the time used by Pratt, and also slightly the memory'
                  'required.'))
parser.add_option('--maxNumFlexSpaces', type=int, help=('Maximum length of flexible spaces. Using this option you can set the'
                  'maximum number of flexible wildcards (matching a variable number of'
                  'arbitrary sequence symbols) (FN parameter). Increasing this will'
                  'increase the time used by Pratt.'))
parser.add_option('--maxFlexibility', type=int, help=('Maximum flexibility. You can set the maximum flexibility of a flexible'
                  'wildcard (matching a variable number of arbitrary sequence symbols)'
                  '(FL parameter). For instance x(2,4) and x(10,12) has flexibility 2,'
                  'and x(10) has flexibility 0. Increasing this will increase the time'
                  'used by Pratt.'))
parser.add_option('--maxFlexProduct', type=int, help=('Maximum flex. product. Using this option you can set an upper limit on'
                  'the product of a flexibilities for a pattern (FP parameter). This is'
                  'related to the memory requirements of the search, and increasing the'
                  'limit, increases the memory usage.'))
parser.add_option('--patternSymbolFile', action='store_true', help=('Pattern Symbol File (BI parameter)'))
parser.add_option('--numPatternSymbols', type=int, help=('Number of pattern symbols used in the initial search (BN parameter).'))
parser.add_option('--patternScoring', type=str, help=('Pattern scoring (S parameter)'))
parser.add_option('--patternGraph', type=str, help=('Pattern Graph (G parameter) allows the use of an alignment or a query'
                  'sequence to restrict the pattern search.'))
parser.add_option('--searchGreediness', type=int, help=('Using the greediness parameter (E) you can adjust the greediness of'
                  'the search. Setting E to 0 (zero), the search will be exhaustive.'
                  'Increasing E increases the greediness, and decreases the time used in'
                  'the search.'))
parser.add_option('--patternRefinement', action='store_true', help=('Pattern Refinement (R parameter). When the R option is switched on,'
                  'patterns found during the initial pattern search are input to a'
                  'refinement algorithm where more ambiguous pattern symbols can be'
                  'added.'))
parser.add_option('--genAmbigSymbols', action='store_true', help=('Generalise ambiguous symbols (RG parameter). If the RG option is'
                  'switched on, then ambiguous symbols listed in the symbols file are'
                  'used. If RG is off, only the letters needed to match the input'
                  'sequences are included in the ambiguous pattern positions.'))
parser.add_option('--patternFormat', action='store_true', help=('PROSITE Pattern Format (OP parameter). When switched on, patterns will'
                  'be output in PROSITE style (for instance C-x(2,4)-[DE]). When switched'
                  'off, patterns are output in a simpler consensus pattern style (for'
                  'instance Cxx--[DE] where x matches exactly one arbitrary sequence'
                  'symbol and - matches zero or one arbitrary sequence symbol).'))
parser.add_option('--maxNumPatterns', type=int, help=('Maximum number of patterns (ON parameter) between 1 and 100.'))
parser.add_option('--maxNumAlignments', type=int, help=('Maximum number of alignments (OA parameter) between 1 and 100.'))
parser.add_option('--printPatterns', action='store_true', help=('Print Patterns in sequences (M parameter) If the M option is set, then'
                  'Pratt will print out the location of the sequence segments matching'
                  'each of the (maximum 52) best patterns. The patterns are given labels'
                  'A, B,...Z,a,b,...z in order of decreasing pattern score. Each sequence'
                  'is printed on a line, one character per K-tuple in the sequence. If'
                  'pattern with label C matches the third K-tuple in a sequence C is'
                  'printed out. If several patterns match in the same K-tuple, only the'
                  'best will be printed.'))
parser.add_option('--printingRatio', type=int, help=('Printing ratio (MR parameter). sets the K value (ratio) used for'
                  'printing the summary information about where in each sequence the'
                  'pattern matches are found.'))
parser.add_option('--printVertically', action='store_true', help=('Print vertically (MV parameter). if set, the output is printed'
                  'vertically instead of horizontally, vertical output can be better for'
                  'large sequence sets.'))
parser.add_option('--stype', type=str, help=('Defines the type of the sequences to be aligned.'))
parser.add_option('--sequence', type=str, help=('The input set of up to 100 sequences can be entered directly into this'
                  'form. The sequences can be in FASTA or UniProtKB/Swiss-Prot format. A'
                  'partially formatted sequences are not accepted. Note that directly'
                  'using data from word processors may yield unpredictable results as'
                  'hidden/control characters may be present.'))
parser.add_option('--ppfile', type=str, help=('Pattern restriction file. The restriction file limits the sequence'
                  'range via the start/end parameter and is in the format >Sequence'
                  '(start, end). If parameter PP is off, the restiction file will be'
                  'ignored.'))
# General options
parser.add_option('-h', '--help', action='store_true', help='Show this help message and exit.')
parser.add_option('--email', help='E-mail address.')
parser.add_option('--title', help='Job title.')
parser.add_option('--outfile', help='File name for results.')
parser.add_option('--outformat', help='Output format for results.')
parser.add_option('--asyncjob', action='store_true', help='Asynchronous mode.')
parser.add_option('--jobid', help='Job identifier.')
parser.add_option('--polljob', action="store_true", help='Get job result.')
parser.add_option('--pollFreq', type='int', default=3, help='Poll frequency in seconds (default 3s).')
parser.add_option('--status', action="store_true", help='Get job status.')
parser.add_option('--resultTypes', action='store_true', help='Get result types.')
parser.add_option('--params', action='store_true', help='List input parameters.')
parser.add_option('--paramDetail', help='Get details for parameter.')
parser.add_option('--multifasta', action='store_true', help='Treat input as a set of fasta formatted sequences.')
parser.add_option('--useSeqId', action='store_true', help='Use sequence identifiers for output filenames.'
                                                          'Only available in multi-fasta and multi-identifier modes.')
parser.add_option('--maxJobs', type='int', help='Maximum number of concurrent jobs. '
                                                'Only available in multifasta or list file modes.')

parser.add_option('--quiet', action='store_true', help='Decrease output level.')
parser.add_option('--verbose', action='store_true', help='Increase output level.')
parser.add_option('--version', action='store_true', help='Prints out the version of the Client and exit.')
parser.add_option('--debugLevel', type='int', default=debugLevel, help='Debugging level.')
parser.add_option('--baseUrl', default=baseUrl, help='Base URL for service.')

(options, args) = parser.parse_args()

# Increase output level
if options.verbose:
    outputLevel += 1

# Decrease output level
if options.quiet:
    outputLevel -= 1

# Debug level
if options.debugLevel:
    debugLevel = options.debugLevel

if options.pollFreq:
    pollFreq = options.pollFreq

if options.baseUrl:
    baseUrl = options.baseUrl
if options.multifasta:
    multifasta = options.multifasta

if options.useSeqId:
    useSeqId = options.useSeqId

if options.maxJobs:
    maxJobs = options.maxJobs


# Debug print
def printDebugMessage(functionName, message, level):
    if (level <= debugLevel):
        print(u'[' + functionName + u'] ' + message, file=sys.stderr)


# User-agent for request (see RFC2616).
def getUserAgent():
    printDebugMessage(u'getUserAgent', u'Begin', 11)
    # Agent string for urllib2 library.
    urllib_agent = u'Python-urllib/%s' % urllib_version
    clientRevision = version
    # Prepend client specific agent string.
    try:
        pythonversion = platform.python_version()
        pythonsys = platform.system()
    except ValueError:
        pythonversion, pythonsys = "Unknown", "Unknown"
    user_agent = u'EBI-Sample-Client/%s (%s; Python %s; %s) %s' % (
        clientRevision, os.path.basename(__file__),
        pythonversion, pythonsys, urllib_agent)
    printDebugMessage(u'getUserAgent', u'user_agent: ' + user_agent, 12)
    printDebugMessage(u'getUserAgent', u'End', 11)
    return user_agent


# Wrapper for a REST (HTTP GET) request
def restRequest(url):
    printDebugMessage(u'restRequest', u'Begin', 11)
    printDebugMessage(u'restRequest', u'url: ' + url, 11)
    try:
        # Set the User-agent.
        user_agent = getUserAgent()
        http_headers = {u'User-Agent': user_agent}
        req = Request(url, None, http_headers)
        # Make the request (HTTP GET).
        reqH = urlopen(req)
        resp = reqH.read()
        contenttype = reqH.info()

        if (len(resp) > 0 and contenttype != u"image/png;charset=UTF-8"
                and contenttype != u"image/jpeg;charset=UTF-8"
                and contenttype != u"application/gzip;charset=UTF-8"):
            try:
                result = unicode(resp, u'utf-8')
            except UnicodeDecodeError:
                result = resp
        else:
            result = resp
        reqH.close()
    # Errors are indicated by HTTP status codes.
    except HTTPError as ex:
        result = requests.get(url).content
    printDebugMessage(u'restRequest', u'End', 11)
    return result


# Get input parameters list
def serviceGetParameters():
    printDebugMessage(u'serviceGetParameters', u'Begin', 1)
    requestUrl = baseUrl + u'/parameters'
    printDebugMessage(u'serviceGetParameters', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetParameters', u'End', 1)
    return doc[u'id':]


# Print list of parameters
def printGetParameters():
    printDebugMessage(u'printGetParameters', u'Begin', 1)
    idList = serviceGetParameters()
    for id_ in idList:
        print(id_)
    printDebugMessage(u'printGetParameters', u'End', 1)


# Get input parameter information
def serviceGetParameterDetails(paramName):
    printDebugMessage(u'serviceGetParameterDetails', u'Begin', 1)
    printDebugMessage(u'serviceGetParameterDetails', u'paramName: ' + paramName, 2)
    requestUrl = baseUrl + u'/parameterdetails/' + paramName
    printDebugMessage(u'serviceGetParameterDetails', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetParameterDetails', u'End', 1)
    return doc


# Print description of a parameter
def printGetParameterDetails(paramName):
    printDebugMessage(u'printGetParameterDetails', u'Begin', 1)
    doc = serviceGetParameterDetails(paramName)
    print(unicode(doc.name) + u"\t" + unicode(doc.type))
    print(doc.description)
    if hasattr(doc, 'values'):
        for value in doc.values:
            print(value.value)
            if unicode(value.defaultValue) == u'true':
                print(u'default')
            print(u"\t" + unicode(value.label))
            if hasattr(value, u'properties'):
                for wsProperty in value.properties:
                    print(u"\t" + unicode(wsProperty.key) + u"\t" + unicode(wsProperty.value))
    printDebugMessage(u'printGetParameterDetails', u'End', 1)


# Submit job
def serviceRun(email, title, params):
    printDebugMessage(u'serviceRun', u'Begin', 1)
    # Insert e-mail and title into params
    params[u'email'] = email
    if title:
        params[u'title'] = title
    requestUrl = baseUrl + u'/run/'
    printDebugMessage(u'serviceRun', u'requestUrl: ' + requestUrl, 2)

    # Get the data for the other options
    requestData = urlencode(params)

    printDebugMessage(u'serviceRun', u'requestData: ' + requestData, 2)
    # Errors are indicated by HTTP status codes.
    try:
        # Set the HTTP User-agent.
        user_agent = getUserAgent()
        http_headers = {u'User-Agent': user_agent}
        req = Request(requestUrl, None, http_headers)
        # Make the submission (HTTP POST).
        reqH = urlopen(req, requestData.encode(encoding=u'utf_8', errors=u'strict'))
        jobId = unicode(reqH.read(), u'utf-8')
        reqH.close()
    except HTTPError as ex:
        print(xmltramp.parse(unicode(ex.read(), u'utf-8'))[0][0])
        quit()
    printDebugMessage(u'serviceRun', u'jobId: ' + jobId, 2)
    printDebugMessage(u'serviceRun', u'End', 1)
    return jobId
def multipleServiceRun(email, title, params, useSeqId, maxJobs, outputLevel):
    seqs = params['sequence']
    seqs = seqs.split(">")[1:]
    i = 0
    j = maxJobs
    done = 0
    jobs = []
    while done < len(seqs):
        c = 0
        for seq in seqs[i:j]:
            c += 1
            params['sequence'] = ">" + seq
            if c <= int(maxJobs):
                jobId = serviceRun(options.email, options.title, params)
                jobs.append(jobId)
                if outputLevel > 0:
                    if useSeqId:
                        print("Submitting job for: %s" % str(seq.split()[0]))
                    else:
                        print("JobId: " + jobId, file=sys.stderr)
        for k, jobId in enumerate(jobs[:]):
            if outputLevel > 0:
                print("JobId: " + jobId, file=sys.stderr)
            else:
                print(jobId)
            if useSeqId:
                options.outfile = str(seqs[i + k].split()[0])
            getResult(jobId)
            done += 1
            jobs.remove(jobId)
        i += maxJobs
        j += maxJobs
        time.sleep(pollFreq)



# Get job status
def serviceGetStatus(jobId):
    printDebugMessage(u'serviceGetStatus', u'Begin', 1)
    printDebugMessage(u'serviceGetStatus', u'jobId: ' + jobId, 2)
    requestUrl = baseUrl + u'/status/' + jobId
    printDebugMessage(u'serviceGetStatus', u'requestUrl: ' + requestUrl, 2)
    status = restRequest(requestUrl)
    printDebugMessage(u'serviceGetStatus', u'status: ' + status, 2)
    printDebugMessage(u'serviceGetStatus', u'End', 1)
    return status


# Print the status of a job
def printGetStatus(jobId):
    printDebugMessage(u'printGetStatus', u'Begin', 1)
    status = serviceGetStatus(jobId)
    if outputLevel > 0:
        print("Getting status for job %s" % jobId)
    print(status)
    if outputLevel > 0 and status == "FINISHED":
        print("To get results: python %s --polljob --jobid %s"
              "" % (os.path.basename(__file__), jobId))
    printDebugMessage(u'printGetStatus', u'End', 1)


# Get available result types for job
def serviceGetResultTypes(jobId):
    printDebugMessage(u'serviceGetResultTypes', u'Begin', 1)
    printDebugMessage(u'serviceGetResultTypes', u'jobId: ' + jobId, 2)
    requestUrl = baseUrl + u'/resulttypes/' + jobId
    printDebugMessage(u'serviceGetResultTypes', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetResultTypes', u'End', 1)
    return doc[u'type':]


# Print list of available result types for a job.
def printGetResultTypes(jobId):
    printDebugMessage(u'printGetResultTypes', u'Begin', 1)
    if outputLevel > 0:
        print("Getting result types for job %s" % jobId)

    resultTypeList = serviceGetResultTypes(jobId)
    if outputLevel > 0:
        print("Available result types:")
    for resultType in resultTypeList:
        print(resultType[u'identifier'])
        if hasattr(resultType, u'label'):
            print(u"\t", resultType[u'label'])
        if hasattr(resultType, u'description'):
            print(u"\t", resultType[u'description'])
        if hasattr(resultType, u'mediaType'):
            print(u"\t", resultType[u'mediaType'])
        if hasattr(resultType, u'fileSuffix'):
            print(u"\t", resultType[u'fileSuffix'])
    if outputLevel > 0:
        print("To get results:\n  python %s --polljob --jobid %s\n"
              "  python %s --polljob --outformat <type> --jobid %s"
              "" % (os.path.basename(__file__), jobId,
                    os.path.basename(__file__), jobId))
    printDebugMessage(u'printGetResultTypes', u'End', 1)


# Get result
def serviceGetResult(jobId, type_):
    printDebugMessage(u'serviceGetResult', u'Begin', 1)
    printDebugMessage(u'serviceGetResult', u'jobId: ' + jobId, 2)
    printDebugMessage(u'serviceGetResult', u'type_: ' + type_, 2)
    requestUrl = baseUrl + u'/result/' + jobId + u'/' + type_
    result = restRequest(requestUrl)
    printDebugMessage(u'serviceGetResult', u'End', 1)
    return result


# Client-side poll
def clientPoll(jobId):
    printDebugMessage(u'clientPoll', u'Begin', 1)
    result = u'PENDING'
    while result == u'RUNNING' or result == u'PENDING':
        result = serviceGetStatus(jobId)
        if outputLevel > 0:
            print(result)
        if result == u'RUNNING' or result == u'PENDING':
            time.sleep(pollFreq)
    printDebugMessage(u'clientPoll', u'End', 1)


# Get result for a jobid
# Allows more than one output file written when 'outformat' is defined.
def getResult(jobId):
    printDebugMessage(u'getResult', u'Begin', 1)
    printDebugMessage(u'getResult', u'jobId: ' + jobId, 1)
    if outputLevel > 1:
        print("Getting results for job %s" % jobId)
    # Check status and wait if necessary
    clientPoll(jobId)
    # Get available result types
    resultTypes = serviceGetResultTypes(jobId)

    for resultType in resultTypes:
        # Derive the filename for the result
        if options.outfile:
            filename = (options.outfile + u'.' + unicode(resultType[u'identifier']) +
                        u'.' + unicode(resultType[u'fileSuffix']))
        else:
            filename = (jobId + u'.' + unicode(resultType[u'identifier']) +
                        u'.' + unicode(resultType[u'fileSuffix']))
        # Write a result file

        outformat_parm = str(options.outformat).split(',')
        for outformat_type in outformat_parm:
            outformat_type = outformat_type.replace(' ', '')

            if outformat_type == 'None':
                outformat_type = None

            if not outformat_type or outformat_type == unicode(resultType[u'identifier']):
                if outputLevel > 1:
                    print("Getting %s" % unicode(resultType[u'identifier']))
                # Get the result
                result = serviceGetResult(jobId, unicode(resultType[u'identifier']))
                if (unicode(resultType[u'mediaType']) == u"image/png"
                        or unicode(resultType[u'mediaType']) == u"image/jpeg"
                        or unicode(resultType[u'mediaType']) == u"application/gzip"):
                    fmode = 'wb'
                else:
                    fmode = 'w'

                try:
                    fh = open(filename, fmode)
                    fh.write(result)
                    fh.close()
                except TypeError:
                    fh.close()
                    fh = open(filename, "wb")
                    fh.write(result)
                    fh.close()
                if outputLevel > 0:
                    print("Creating result file: " + filename)
    printDebugMessage(u'getResult', u'End', 1)


# Read a file
def readFile(filename):
    printDebugMessage(u'readFile', u'Begin', 1)
    fh = open(filename, 'r')
    data = fh.read()
    fh.close()
    printDebugMessage(u'readFile', u'End', 1)
    return data


def print_usage():
    print("""\
EMBL-EBI PRATT Python Client:

Protein function analysis with Pratt.

[Required (for job submission)]
  --email               E-mail address.
  --stype               Defines the type of the sequences to be aligned.
  --sequence            The input set of up to 100 sequences can be entered directly
                        into this form. The sequences can be in FASTA or
                        UniProtKB/Swiss-Prot format. A partially formatted sequences
                        are not accepted. Note that directly using data from word
                        processors may yield unpredictable results as hidden/control
                        characters may be present.

[Optional]
  --minPerc             Set the minimum percentage of the input sequences that
                        should match a pattern (C%). If you set this to, say 80,
                        Pratt will only report patterns matching at least 80 % of
                        the sequences input.
  --patternPosition     Pattern position in sequence (PP parameter).
  --maxPatternLength    Maximum pattern length (PL parameter) allows you to set the
                        maximum length of a pattern. The length of the pattern
                        C-x(2,4)-[DE] is 1+4+1=6. The memory requirement of Pratt
                        depends on L; a higher L value gives higher memory
                        requirement.
  --maxNumPatternSymbols Maximum number of pattern symbols (PN parameter). Using this
                        you can set the maximum number of symbols in a pattern. The
                        pattern C-x(2,4)-[DE] has 2 symbols (C and [DE]). When PN is
                        increased, Pratt will require more memory.
  --maxNumWildcard      Maximum length of a widecard (x). Using this option you can
                        set the maximum length of a wildcard (PX parameter).
                        Increasing this will increase the time used by Pratt, and
                        also slightly the memory required.
  --maxNumFlexSpaces    Maximum length of flexible spaces. Using this option you can
                        set the maximum number of flexible wildcards (matching a
                        variable number of arbitrary sequence symbols) (FN
                        parameter). Increasing this will increase the time used by
                        Pratt.
  --maxFlexibility      Maximum flexibility. You can set the maximum flexibility of
                        a flexible wildcard (matching a variable number of arbitrary
                        sequence symbols) (FL parameter). For instance x(2,4) and
                        x(10,12) has flexibility 2, and x(10) has flexibility 0.
                        Increasing this will increase the time used by Pratt.
  --maxFlexProduct      Maximum flex. product. Using this option you can set an
                        upper limit on the product of a flexibilities for a pattern
                        (FP parameter). This is related to the memory requirements
                        of the search, and increasing the limit, increases the
                        memory usage.
  --patternSymbolFile   Pattern Symbol File (BI parameter).
  --numPatternSymbols   Number of pattern symbols used in the initial search (BN
                        parameter).
  --patternScoring      Pattern scoring (S parameter).
  --patternGraph        Pattern Graph (G parameter) allows the use of an alignment
                        or a query sequence to restrict the pattern search.
  --searchGreediness    Using the greediness parameter (E) you can adjust the
                        greediness of the search. Setting E to 0 (zero), the search
                        will be exhaustive. Increasing E increases the greediness,
                        and decreases the time used in the search.
  --patternRefinement   Pattern Refinement (R parameter). When the R option is
                        switched on, patterns found during the initial pattern
                        search are input to a refinement algorithm where more
                        ambiguous pattern symbols can be added.
  --genAmbigSymbols     Generalise ambiguous symbols (RG parameter). If the RG
                        option is switched on, then ambiguous symbols listed in the
                        symbols file are used. If RG is off, only the letters needed
                        to match the input sequences are included in the ambiguous
                        pattern positions.
  --patternFormat       PROSITE Pattern Format (OP parameter). When switched on,
                        patterns will be output in PROSITE style (for instance
                        C-x(2,4)-[DE]). When switched off, patterns are output in a
                        simpler consensus pattern style (for instance Cxx--[DE]
                        where x matches exactly one arbitrary sequence symbol and -
                        matches zero or one arbitrary sequence symbol).
  --maxNumPatterns      Maximum number of patterns (ON parameter) between 1 and 100.
  --maxNumAlignments    Maximum number of alignments (OA parameter) between 1 and
                        100.
  --printPatterns       Print Patterns in sequences (M parameter) If the M option is
                        set, then Pratt will print out the location of the sequence
                        segments matching each of the (maximum 52) best patterns.
                        The patterns are given labels A, B,...Z,a,b,...z in order of
                        decreasing pattern score. Each sequence is printed on a
                        line, one character per K-tuple in the sequence. If pattern
                        with label C matches the third K-tuple in a sequence C is
                        printed out. If several patterns match in the same K-tuple,
                        only the best will be printed.
  --printingRatio       Printing ratio (MR parameter). sets the K value (ratio) used
                        for printing the summary information about where in each
                        sequence the pattern matches are found.
  --printVertically     Print vertically (MV parameter). if set, the output is
                        printed vertically instead of horizontally, vertical output
                        can be better for large sequence sets.
  --ppfile              Pattern restriction file. The restriction file limits the
                        sequence range via the start/end parameter and is in the
                        format '>Sequence (start, end)'. If parameter PP is off, the
                        restiction file will be ignored.

[General]
  -h, --help            Show this help message and exit.
  --asyncjob            Forces to make an asynchronous query.
  --title               Title for job.
  --status              Get job status.
  --resultTypes         Get available result types for job.
  --polljob             Poll for the status of a job.
  --pollFreq            Poll frequency in seconds (default 3s).
  --jobid               JobId that was returned when an asynchronous job was submitted.
  --outfile             File name for results (default is JobId; for STDOUT).
  --outformat           Result format(s) to retrieve. It accepts comma-separated values.
  --params              List input parameters.
  --paramDetail         Display details for input parameter.
  --verbose             Increase output.
  --version             Prints out the version of the Client and exit.
  --quiet               Decrease output.
  --baseUrl             Base URL. Defaults to:
                        https://www.ebi.ac.uk/Tools/services/rest/pratt

Synchronous job:
  The results/errors are returned as soon as the job is finished.
  Usage: python pratt.py --email <your@email.com> [options...] <SeqFile|SeqID(s)>
  Returns: results as an attachment

Asynchronous job:
  Use this if you want to retrieve the results at a later time. The results
  are stored for up to 24 hours.
  Usage: python pratt.py --asyncjob --email <your@email.com> [options...] <SeqFile|SeqID(s)>
  Returns: jobid

Check status of Asynchronous job:
  Usage: python pratt.py --status --jobid <jobId>

Retrieve job data:
  Use the jobid to query for the status of the job. If the job is finished,
  it also returns the results/errors.
  Usage: python pratt.py --polljob --jobid <jobId> [--outfile string]
  Returns: string indicating the status of the job and if applicable, results
  as an attachment.

Further information:
  https://www.ebi.ac.uk/Tools/webservices and
    https://github.com/ebi-wp/webservice-clients

Support/Feedback:
  https://www.ebi.ac.uk/support/""")


# No options... print help.
if numOpts < 2:
    print_usage()
elif options.help:
    print_usage()
# List parameters
elif options.params:
    printGetParameters()
# Get parameter details
elif options.paramDetail:
    printGetParameterDetails(options.paramDetail)
# Print Client version
elif options.version:
    print("Revision: %s" % version)
    sys.exit()
# Submit job
elif options.email and not options.jobid:
    params = {}
    if len(args) == 1 and "true" not in args and "false" not in args:
        if os.path.exists(args[0]):  # Read file into content
            params[u'sequence'] = readFile(args[0])
        else:  # Argument is a sequence id
            params[u'sequence'] = args[0]
    elif len(args) == 2 and "true" not in args and "false" not in args:
        if os.path.exists(args[0]) and os.path.exists(args[1]):  # Read file into content
            params[u'asequence'] = readFile(args[0])
            params[u'bsequence'] = readFile(args[1])
        else:  # Argument is a sequence id
            params[u'asequence'] = args[0]
            params[u'bsequence'] = args[0]
    elif hasattr(options, "sequence") or (hasattr(options, "asequence") and hasattr(options, "bsequence")):  # Specified via option
        if hasattr(options, "sequence"):
            if os.path.exists(options.sequence):  # Read file into content
                params[u'sequence'] = readFile(options.sequence)
            else:  # Argument is a sequence id
                params[u'sequence'] = options.sequence
        elif hasattr(options, "asequence") and hasattr(options, "bsequence"):
            if os.path.exists(options.asequence) and os.path.exists(options.bsequence):  # Read file into content
                params[u'asequence'] = readFile(options.asequence)
                params[u'bsequence'] = readFile(options.bsequence)
            else:  # Argument is a sequence id
                params[u'asequence'] = options.asequence
                params[u'bsequence'] = options.bsequence

    # Pass default values and fix bools (without default value)
    if options.stype:
        params['stype'] = options.stype

    if options.minPerc:
        params['minPerc'] = options.minPerc
    

    if not options.patternPosition:
        params['patternPosition'] = 'off'
    if options.patternPosition:
        params['patternPosition'] = options.patternPosition
    

    if options.maxPatternLength:
        params['maxPatternLength'] = options.maxPatternLength
    

    if options.maxNumPatternSymbols:
        params['maxNumPatternSymbols'] = options.maxNumPatternSymbols
    

    if options.maxNumWildcard:
        params['maxNumWildcard'] = options.maxNumWildcard
    

    if options.maxNumFlexSpaces:
        params['maxNumFlexSpaces'] = options.maxNumFlexSpaces
    

    if options.maxFlexibility:
        params['maxFlexibility'] = options.maxFlexibility
    

    if options.maxFlexProduct:
        params['maxFlexProduct'] = options.maxFlexProduct
    

    if not options.patternSymbolFile:
        params['patternSymbolFile'] = 'false'
    if options.patternSymbolFile:
        params['patternSymbolFile'] = options.patternSymbolFile
    

    if options.numPatternSymbols:
        params['numPatternSymbols'] = options.numPatternSymbols
    

    if not options.patternScoring:
        params['patternScoring'] = 'info'
    if options.patternScoring:
        params['patternScoring'] = options.patternScoring
    

    if options.patternGraph:
        params['patternGraph'] = options.patternGraph
    

    if options.searchGreediness:
        params['searchGreediness'] = options.searchGreediness
    

    if not options.patternRefinement:
        params['patternRefinement'] = 'false'
    if options.patternRefinement:
        params['patternRefinement'] = options.patternRefinement
    

    if not options.genAmbigSymbols:
        params['genAmbigSymbols'] = 'false'
    if options.genAmbigSymbols:
        params['genAmbigSymbols'] = options.genAmbigSymbols
    

    if not options.patternFormat:
        params['patternFormat'] = 'true'
    if options.patternFormat:
        params['patternFormat'] = options.patternFormat
    

    if options.maxNumPatterns:
        params['maxNumPatterns'] = options.maxNumPatterns
    

    if options.maxNumAlignments:
        params['maxNumAlignments'] = options.maxNumAlignments
    

    if not options.printPatterns:
        params['printPatterns'] = 'true'
    if options.printPatterns:
        params['printPatterns'] = options.printPatterns
    

    if options.printingRatio:
        params['printingRatio'] = options.printingRatio
    

    if not options.printVertically:
        params['printVertically'] = 'false'
    if options.printVertically:
        params['printVertically'] = options.printVertically
    

    if options.ppfile:
        params['ppfile'] = options.ppfile
    


    # Submit the job
    if options.multifasta:
        multipleServiceRun(options.email, options.title, params,
                           options.useSeqId, options.maxJobs,
                           outputLevel)
    else:
        if options.useSeqId:
            print("Warning: --useSeqId option ignored.")
        if options.maxJobs:
            print("Warning: --maxJobs option ignored.")

        jobId = serviceRun(options.email, options.title, params)
        if options.asyncjob: # Async mode
            print(jobId)
            if outputLevel > 0:
                print("To check status: python %s --status --jobid %s"
                      "" % (os.path.basename(__file__), jobId))
        else:
            # Sync mode
            if outputLevel > 0:
                print("JobId: " + jobId, file=sys.stderr)
            else:
                print(jobId)
            time.sleep(pollFreq)
            getResult(jobId)

# Get job status
elif options.jobid and options.status:
    printGetStatus(options.jobid)

elif options.jobid and (options.resultTypes or options.polljob):
    status = serviceGetStatus(options.jobid)
    if status == 'PENDING' or status == 'RUNNING':
        print("Error: Job status is %s. "
              "To get result types the job must be finished." % status)
        quit()
    # List result types for job
    if options.resultTypes:
        printGetResultTypes(options.jobid)
    # Get results for job
    elif options.polljob:
        getResult(options.jobid)
else:
    # Checks for 'email' parameter
    if not options.email:
        print('\nParameter "--email" is missing in your command. It is required!\n')

    print(u'Error: unrecognised argument combination', file=sys.stderr)
    print_usage()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
#
# Copyright 2012-2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Python Client Automatically generated with:
# https://github.com/ebi-wp/webservice-clients-generator
#
# PSI-Blast (REST) web service Python client using xmltramp2.
#
# For further information see:
# https://www.ebi.ac.uk/Tools/webservices/
#
###############################################################################

from __future__ import print_function

import os
import sys
import time
import requests
import platform
from xmltramp2 import xmltramp
from optparse import OptionParser

try:
    from urllib.parse import urlparse, urlencode
    from urllib.request import urlopen, Request
    from urllib.error import HTTPError
    from urllib.request import __version__ as urllib_version
except ImportError:
    from urlparse import urlparse
    from urllib import urlencode
    from urllib2 import urlopen, Request, HTTPError
    from urllib2 import __version__ as urllib_version

# allow unicode(str) to be used in python 3
try:
    unicode('')
except NameError:
    unicode = str

# Base URL for service
baseUrl = u'https://www.ebi.ac.uk/Tools/services/rest/psiblast'
version = u'2019-07-03 12:51'

# Set interval for checking status
pollFreq = 3
# Output level
outputLevel = 1
# Debug level
debugLevel = 0
# Number of option arguments.
numOpts = len(sys.argv)

# Process command-line options
parser = OptionParser(add_help_option=False)

# Tool specific options (Try to print all the commands automatically)
parser.add_option('--matrix', type=str, help=('The comparison matrix to be used to score alignments when searching'
                  'the database'))
parser.add_option('--gapopen', type=int, help=('Penalty taken away from the score when a gap is created in sequence.'
                  'Increasing the gap openning penalty will decrease the number of gaps'
                  'in the final alignment.'))
parser.add_option('--gapext', type=int, help=('Penalty taken away from the score for each base or residue in the gap.'
                  'Increasing the gap extension penalty favors short gaps in the final'
                  'alignment, conversly decreasing the gap extension penalty favors long'
                  'gaps in the final alignment.'))
parser.add_option('--expthr', type=str, help=('Limits the number of scores and alignments reported based on the'
                  'expectation value. This is the maximum number of times the match is'
                  'expected to occur by chance.'))
parser.add_option('--psithr', type=str, help=('Expectation value threshold for automatic selection of matched'
                  'sequences for inclusion in PSSM at each iteration.'))
parser.add_option('--scores', type=int, help=('Maximum number of match score summaries reported in the result output.'))
parser.add_option('--alignments', type=int, help=('Maximum number of match alignments reported in the result output.'))
parser.add_option('--alignView', type=int, help=('Formating for the alignments'))
parser.add_option('--dropoff', type=int, help=('The amount a score can drop before extension of word hits is halted'))
parser.add_option('--finaldropoff', type=int, help=('Dropoff value for final gapped alignment'))
parser.add_option('--filter', type=str, help=('Filter regions of low sequence complexity. This can avoid issues with'
                  'low complexity sequences where matches are found due to composition'
                  'rather than meaningful sequence similarity. However in some cases'
                  'filtering also masks regions of interest and so should be used with'
                  'caution.'))
parser.add_option('--seqrange', type=str, help=('Specify a range or section of the input sequence to use in the search.'
                  'Example: Specifying 34-89 in an input sequence of total length 100,'
                  'will tell BLAST to only use residues 34 to 89, inclusive.'))
parser.add_option('--sequence', type=str, help=('The query sequence can be entered directly into this form. The'
                  'sequence can be in GCG, FASTA, PIR, NBRF, PHYLIP or UniProtKB/Swiss-'
                  'Prot format. A partially formatted sequence is not accepted. Adding a'
                  'return to the end of the sequence may help certain applications'
                  'understand the input. Note that directly using data from word'
                  'processors may yield unpredictable results as hidden/control'
                  'characters may be present.'))
parser.add_option('--database', type=str, help=('The databases to run the sequence similarity search against. Multiple'
                  'databases can be used at the same time'))
parser.add_option('--previousjobid', type=str, help=('The job identifier for the previous PSI-BLAST iteration.'))
parser.add_option('--selectedHits', type=str, help=('List of identifiers of the hits from the previous iteration to use to'
                  'construct the search PSSM for this iteration.'))
parser.add_option('--cpfile', type=str, help=('Checkpoint file from the previous iteration. Must be in ASN.1 Binary'
                  'Format.'))
parser.add_option('--umode', type=str, help=('Usage mode for PHI-BLAST functionality'))
parser.add_option('--patfile', type=str, help=('Pattern file for PHI-BLAST functionality. This file needs to be in the'
                  'style of a prosite entry file, with at least an ID line, PA line and'
                  'optional HI line.'))
# General options
parser.add_option('-h', '--help', action='store_true', help='Show this help message and exit.')
parser.add_option('--email', help='E-mail address.')
parser.add_option('--title', help='Job title.')
parser.add_option('--outfile', help='File name for results.')
parser.add_option('--outformat', help='Output format for results.')
parser.add_option('--asyncjob', action='store_true', help='Asynchronous mode.')
parser.add_option('--jobid', help='Job identifier.')
parser.add_option('--polljob', action="store_true", help='Get job result.')
parser.add_option('--pollFreq', type='int', default=3, help='Poll frequency in seconds (default 3s).')
parser.add_option('--status', action="store_true", help='Get job status.')
parser.add_option('--resultTypes', action='store_true', help='Get result types.')
parser.add_option('--params', action='store_true', help='List input parameters.')
parser.add_option('--paramDetail', help='Get details for parameter.')
parser.add_option('--multifasta', action='store_true', help='Treat input as a set of fasta formatted sequences.')
parser.add_option('--useSeqId', action='store_true', help='Use sequence identifiers for output filenames.'
                                                          'Only available in multi-fasta and multi-identifier modes.')
parser.add_option('--maxJobs', type='int', help='Maximum number of concurrent jobs. '
                                                'Only available in multifasta or list file modes.')

parser.add_option('--quiet', action='store_true', help='Decrease output level.')
parser.add_option('--verbose', action='store_true', help='Increase output level.')
parser.add_option('--version', action='store_true', help='Prints out the version of the Client and exit.')
parser.add_option('--debugLevel', type='int', default=debugLevel, help='Debugging level.')
parser.add_option('--baseUrl', default=baseUrl, help='Base URL for service.')

(options, args) = parser.parse_args()

# Increase output level
if options.verbose:
    outputLevel += 1

# Decrease output level
if options.quiet:
    outputLevel -= 1

# Debug level
if options.debugLevel:
    debugLevel = options.debugLevel

if options.pollFreq:
    pollFreq = options.pollFreq

if options.baseUrl:
    baseUrl = options.baseUrl
if options.multifasta:
    multifasta = options.multifasta

if options.useSeqId:
    useSeqId = options.useSeqId

if options.maxJobs:
    maxJobs = options.maxJobs


# Debug print
def printDebugMessage(functionName, message, level):
    if (level <= debugLevel):
        print(u'[' + functionName + u'] ' + message, file=sys.stderr)


# User-agent for request (see RFC2616).
def getUserAgent():
    printDebugMessage(u'getUserAgent', u'Begin', 11)
    # Agent string for urllib2 library.
    urllib_agent = u'Python-urllib/%s' % urllib_version
    clientRevision = version
    # Prepend client specific agent string.
    try:
        pythonversion = platform.python_version()
        pythonsys = platform.system()
    except ValueError:
        pythonversion, pythonsys = "Unknown", "Unknown"
    user_agent = u'EBI-Sample-Client/%s (%s; Python %s; %s) %s' % (
        clientRevision, os.path.basename(__file__),
        pythonversion, pythonsys, urllib_agent)
    printDebugMessage(u'getUserAgent', u'user_agent: ' + user_agent, 12)
    printDebugMessage(u'getUserAgent', u'End', 11)
    return user_agent


# Wrapper for a REST (HTTP GET) request
def restRequest(url):
    printDebugMessage(u'restRequest', u'Begin', 11)
    printDebugMessage(u'restRequest', u'url: ' + url, 11)
    try:
        # Set the User-agent.
        user_agent = getUserAgent()
        http_headers = {u'User-Agent': user_agent}
        req = Request(url, None, http_headers)
        # Make the request (HTTP GET).
        reqH = urlopen(req)
        resp = reqH.read()
        contenttype = reqH.info()

        if (len(resp) > 0 and contenttype != u"image/png;charset=UTF-8"
                and contenttype != u"image/jpeg;charset=UTF-8"
                and contenttype != u"application/gzip;charset=UTF-8"):
            try:
                result = unicode(resp, u'utf-8')
            except UnicodeDecodeError:
                result = resp
        else:
            result = resp
        reqH.close()
    # Errors are indicated by HTTP status codes.
    except HTTPError as ex:
        result = requests.get(url).content
    printDebugMessage(u'restRequest', u'End', 11)
    return result


# Get input parameters list
def serviceGetParameters():
    printDebugMessage(u'serviceGetParameters', u'Begin', 1)
    requestUrl = baseUrl + u'/parameters'
    printDebugMessage(u'serviceGetParameters', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetParameters', u'End', 1)
    return doc[u'id':]


# Print list of parameters
def printGetParameters():
    printDebugMessage(u'printGetParameters', u'Begin', 1)
    idList = serviceGetParameters()
    for id_ in idList:
        print(id_)
    printDebugMessage(u'printGetParameters', u'End', 1)


# Get input parameter information
def serviceGetParameterDetails(paramName):
    printDebugMessage(u'serviceGetParameterDetails', u'Begin', 1)
    printDebugMessage(u'serviceGetParameterDetails', u'paramName: ' + paramName, 2)
    requestUrl = baseUrl + u'/parameterdetails/' + paramName
    printDebugMessage(u'serviceGetParameterDetails', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetParameterDetails', u'End', 1)
    return doc


# Print description of a parameter
def printGetParameterDetails(paramName):
    printDebugMessage(u'printGetParameterDetails', u'Begin', 1)
    doc = serviceGetParameterDetails(paramName)
    print(unicode(doc.name) + u"\t" + unicode(doc.type))
    print(doc.description)
    if hasattr(doc, 'values'):
        for value in doc.values:
            print(value.value)
            if unicode(value.defaultValue) == u'true':
                print(u'default')
            print(u"\t" + unicode(value.label))
            if hasattr(value, u'properties'):
                for wsProperty in value.properties:
                    print(u"\t" + unicode(wsProperty.key) + u"\t" + unicode(wsProperty.value))
    printDebugMessage(u'printGetParameterDetails', u'End', 1)


# Submit job
def serviceRun(email, title, params):
    printDebugMessage(u'serviceRun', u'Begin', 1)
    # Insert e-mail and title into params
    params[u'email'] = email
    if title:
        params[u'title'] = title
    requestUrl = baseUrl + u'/run/'
    printDebugMessage(u'serviceRun', u'requestUrl: ' + requestUrl, 2)

    # Get the data for the other options
    requestData = urlencode(params)

    printDebugMessage(u'serviceRun', u'requestData: ' + requestData, 2)
    # Errors are indicated by HTTP status codes.
    try:
        # Set the HTTP User-agent.
        user_agent = getUserAgent()
        http_headers = {u'User-Agent': user_agent}
        req = Request(requestUrl, None, http_headers)
        # Make the submission (HTTP POST).
        reqH = urlopen(req, requestData.encode(encoding=u'utf_8', errors=u'strict'))
        jobId = unicode(reqH.read(), u'utf-8')
        reqH.close()
    except HTTPError as ex:
        print(xmltramp.parse(unicode(ex.read(), u'utf-8'))[0][0])
        quit()
    printDebugMessage(u'serviceRun', u'jobId: ' + jobId, 2)
    printDebugMessage(u'serviceRun', u'End', 1)
    return jobId
def multipleServiceRun(email, title, params, useSeqId, maxJobs, outputLevel):
    seqs = params['sequence']
    seqs = seqs.split(">")[1:]
    i = 0
    j = maxJobs
    done = 0
    jobs = []
    while done < len(seqs):
        c = 0
        for seq in seqs[i:j]:
            c += 1
            params['sequence'] = ">" + seq
            if c <= int(maxJobs):
                jobId = serviceRun(options.email, options.title, params)
                jobs.append(jobId)
                if outputLevel > 0:
                    if useSeqId:
                        print("Submitting job for: %s" % str(seq.split()[0]))
                    else:
                        print("JobId: " + jobId, file=sys.stderr)
        for k, jobId in enumerate(jobs[:]):
            if outputLevel > 0:
                print("JobId: " + jobId, file=sys.stderr)
            else:
                print(jobId)
            if useSeqId:
                options.outfile = str(seqs[i + k].split()[0])
            getResult(jobId)
            done += 1
            jobs.remove(jobId)
        i += maxJobs
        j += maxJobs
        time.sleep(pollFreq)



# Get job status
def serviceGetStatus(jobId):
    printDebugMessage(u'serviceGetStatus', u'Begin', 1)
    printDebugMessage(u'serviceGetStatus', u'jobId: ' + jobId, 2)
    requestUrl = baseUrl + u'/status/' + jobId
    printDebugMessage(u'serviceGetStatus', u'requestUrl: ' + requestUrl, 2)
    status = restRequest(requestUrl)
    printDebugMessage(u'serviceGetStatus', u'status: ' + status, 2)
    printDebugMessage(u'serviceGetStatus', u'End', 1)
    return status


# Print the status of a job
def printGetStatus(jobId):
    printDebugMessage(u'printGetStatus', u'Begin', 1)
    status = serviceGetStatus(jobId)
    if outputLevel > 0:
        print("Getting status for job %s" % jobId)
    print(status)
    if outputLevel > 0 and status == "FINISHED":
        print("To get results: python %s --polljob --jobid %s"
              "" % (os.path.basename(__file__), jobId))
    printDebugMessage(u'printGetStatus', u'End', 1)


# Get available result types for job
def serviceGetResultTypes(jobId):
    printDebugMessage(u'serviceGetResultTypes', u'Begin', 1)
    printDebugMessage(u'serviceGetResultTypes', u'jobId: ' + jobId, 2)
    requestUrl = baseUrl + u'/resulttypes/' + jobId
    printDebugMessage(u'serviceGetResultTypes', u'requestUrl: ' + requestUrl, 2)
    xmlDoc = restRequest(requestUrl)
    doc = xmltramp.parse(xmlDoc)
    printDebugMessage(u'serviceGetResultTypes', u'End', 1)
    return doc[u'type':]


# Print list of available result types for a job.
def printGetResultTypes(jobId):
    printDebugMessage(u'printGetResultTypes', u'Begin', 1)
    if outputLevel > 0:
        print("Getting result types for job %s" % jobId)

    resultTypeList = serviceGetResultTypes(jobId)
    if outputLevel > 0:
        print("Available result types:")
    for resultType in resultTypeList:
        print(resultType[u'identifier'])
        if hasattr(resultType, u'label'):
            print(u"\t", resultType[u'label'])
        if hasattr(resultType, u'description'):
            print(u"\t", resultType[u'description'])
        if hasattr(resultType, u'mediaType'):
            print(u"\t", resultType[u'mediaType'])
        if hasattr(resultType, u'fileSuffix'):
            print(u"\t", resultType[u'fileSuffix'])
    if outputLevel > 0:
        print("To get results:\n  python %s --polljob --jobid %s\n"
              "  python %s --polljob --outformat <type> --jobid %s"
              "" % (os.path.basename(__file__), jobId,
                    os.path.basename(__file__), jobId))
    printDebugMessage(u'printGetResultTypes', u'End', 1)


# Get result
def serviceGetResult(jobId, type_):
    printDebugMessage(u'serviceGetResult', u'Begin', 1)
    printDebugMessage(u'serviceGetResult', u'jobId: ' + jobId, 2)
    printDebugMessage(u'serviceGetResult', u'type_: ' + type_, 2)
    requestUrl = baseUrl + u'/result/' + jobId + u'/' + type_
    result = restRequest(requestUrl)
    printDebugMessage(u'serviceGetResult', u'End', 1)
    return result


# Client-side poll
def clientPoll(jobId):
    printDebugMessage(u'clientPoll', u'Begin', 1)
    result = u'PENDING'
    while result == u'RUNNING' or result == u'PENDING':
        result = serviceGetStatus(jobId)
        if outputLevel > 0:
            print(result)
        if result == u'RUNNING' or result == u'PENDING':
            time.sleep(pollFreq)
    printDebugMessage(u'clientPoll', u'End', 1)


# Get result for a jobid
# Allows more than one output file written when 'outformat' is defined.
def getResult(jobId):
    printDebugMessage(u'getResult', u'Begin', 1)
    printDebugMessage(u'getResult', u'jobId: ' + jobId, 1)
    if outputLevel > 1:
        print("Getting results for job %s" % jobId)
    # Check status and wait if necessary
    clientPoll(jobId)
    # Get available result types
    resultTypes = serviceGetResultTypes(jobId)

    for resultType in resultTypes:
        # Derive the filename for the result
        if options.outfile:
            filename = (options.outfile + u'.' + unicode(resultType[u'identifier']) +
                        u'.' + unicode(resultType[u'fileSuffix']))
        else:
            filename = (jobId + u'.' + unicode(resultType[u'identifier']) +
                        u'.' + unicode(resultType[u'fileSuffix']))
        # Write a result file

        outformat_parm = str(options.outformat).split(',')
        for outformat_type in outformat_parm:
            outformat_type = outformat_type.replace(' ', '')

            if outformat_type == 'None':
                outformat_type = None

            if not outformat_type or outformat_type == unicode(resultType[u'identifier']):
                if outputLevel > 1:
                    print("Getting %s" % unicode(resultType[u'identifier']))
                # Get the result
                result = serviceGetResult(jobId, unicode(resultType[u'identifier']))
                if (unicode(resultType[u'mediaType']) == u"image/png"
                        or unicode(resultType[u'mediaType']) == u"image/jpeg"
                        or unicode(resultType[u'mediaType']) == u"application/gzip"):
                    fmode = 'wb'
                else:
                    fmode = 'w'

                try:
                    fh = open(filename, fmode)
                    fh.write(result)
                    fh.close()
                except TypeError:
                    fh.close()
                    fh = open(filename, "wb")
                    fh.write(result)
                    fh.close()
                if outputLevel > 0:
                    print("Creating result file: " + filename)
    printDebugMessage(u'getResult', u'End', 1)


# Read a file
def readFile(filename):
    printDebugMessage(u'readFile', u'Begin', 1)
    fh = open(filename, 'r')
    data = fh.read()
    fh.close()
    printDebugMessage(u'readFile', u'End', 1)
    return data


def print_usage():
    print("""\
EMBL-EBI PSI-Blast Python Client:

Sequence similarity search with PSI-Blast.

[Required (for job submission)]
  --email               E-mail address.
  --sequence            The query sequence can be entered directly into this form.
                        The sequence can be in GCG, FASTA, PIR, NBRF, PHYLIP or
                        UniProtKB/Swiss-Prot format. A partially formatted sequence
                        is not accepted. Adding a return to the end of the sequence
                        may help certain applications understand the input. Note
                        that directly using data from word processors may yield
                        unpredictable results as hidden/control characters may be
                        present.
  --database            The databases to run the sequence similarity search against.
                        Multiple databases can be used at the same time.

[Optional]
  --matrix              The comparison matrix to be used to score alignments when
                        searching the database.
  --gapopen             Penalty taken away from the score when a gap is created in
                        sequence. Increasing the gap openning penalty will decrease
                        the number of gaps in the final alignment.
  --gapext              Penalty taken away from the score for each base or residue
                        in the gap. Increasing the gap extension penalty favors
                        short gaps in the final alignment, conversly decreasing the
                        gap extension penalty favors long gaps in the final
                        alignment.
  --expthr              Limits the number of scores and alignments reported based on
                        the expectation value. This is the maximum number of times
                        the match is expected to occur by chance.
  --psithr              Expectation value threshold for automatic selection of
                        matched sequences for inclusion in PSSM at each iteration.
  --scores              Maximum number of match score summaries reported in the
                        result output.
  --alignments          Maximum number of match alignments reported in the result
                        output.
  --alignView           Formating for the alignments.
  --dropoff             The amount a score can drop before extension of word hits is
                        halted.
  --finaldropoff        Dropoff value for final gapped alignment.
  --filter              Filter regions of low sequence complexity. This can avoid
                        issues with low complexity sequences where matches are found
                        due to composition rather than meaningful sequence
                        similarity. However in some cases filtering also masks
                        regions of interest and so should be used with caution.
  --seqrange            Specify a range or section of the input sequence to use in
                        the search. Example: Specifying '34-89' in an input sequence
                        of total length 100, will tell BLAST to only use residues 34
                        to 89, inclusive.
  --previousjobid       The job identifier for the previous PSI-BLAST iteration.
  --selectedHits        List of identifiers of the hits from the previous iteration
                        to use to construct the search PSSM for this iteration.
  --cpfile              Checkpoint file from the previous iteration. Must be in
                        ASN.1 Binary Format.
  --umode               Usage mode for PHI-BLAST functionality.
  --patfile             Pattern file for PHI-BLAST functionality. This file needs to
                        be in the style of a prosite entry file, with at least an ID
                        line, PA line and optional HI line.

[General]
  -h, --help            Show this help message and exit.
  --asyncjob            Forces to make an asynchronous query.
  --title               Title for job.
  --status              Get job status.
  --resultTypes         Get available result types for job.
  --polljob             Poll for the status of a job.
  --pollFreq            Poll frequency in seconds (default 3s).
  --jobid               JobId that was returned when an asynchronous job was submitted.
  --outfile             File name for results (default is JobId; for STDOUT).
  --outformat           Result format(s) to retrieve. It accepts comma-separated values.
  --params              List input parameters.
  --paramDetail         Display details for input parameter.
  --verbose             Increase output.
  --version             Prints out the version of the Client and exit.
  --quiet               Decrease output.
  --baseUrl             Base URL. Defaults to:
                        https://www.ebi.ac.uk/Tools/services/rest/psiblast

Synchronous job:
  The results/errors are returned as soon as the job is finished.
  Usage: python psiblast.py --email <your@email.com> [options...] <SeqFile|SeqID(s)>
  Returns: results as an attachment

Asynchronous job:
  Use this if you want to retrieve the results at a later time. The results
  are stored for up to 24 hours.
  Usage: python psiblast.py --asyncjob --email <your@email.com> [options...] <SeqFile|SeqID(s)>
  Returns: jobid

Check status of Asynchronous job:
  Usage: python psiblast.py --status --jobid <jobId>

Retrieve job data:
  Use the jobid to query for the status of the job. If the job is finished,
  it also returns the results/errors.
  Usage: python psiblast.py --polljob --jobid <jobId> [--outfile string]
  Returns: string indicating the status of the job and if applicable, results
  as an attachment.

Further information:
  https://www.ebi.ac.uk/Tools/webservices and
    https://github.com/ebi-wp/webservice-clients

Support/Feedback:
  https://www.ebi.ac.uk/support/""")


# No options... print help.
if numOpts < 2:
    print_usage()
elif options.help:
    print_usage()
# List parameters
elif options.params:
    printGetParameters()
# Get parameter details
elif options.paramDetail:
    printGetParameterDetails(options.paramDetail)
# Print Client version
elif options.version:
    print("Revision: %s" % version)
    sys.exit()
# Submit job
elif options.email and not options.jobid:
    params = {}
    if len(args) == 1 and "true" not in args and "false" not in args:
        if os.path.exists(args[0]):  # Read file into content
            params[u'sequence'] = readFile(args[0])
        else:  # Argument is a sequence id
            params[u'sequence'] = args[0]
    elif len(args) == 2 and "true" not in args and "false" not in args:
        if os.path.exists(args[0]) and os.path.exists(args[1]):  # Read file into content
            params[u'asequence'] = readFile(args[0])
            params[u'bsequence'] = readFile(args[1])
        else:  # Argument is a sequence id
            params[u'asequence'] = args[0]
            params[u'bsequence'] = args[0]
    elif hasattr(options, "sequence") or (hasattr(options, "asequence") and hasattr(options, "bsequence")):  # Specified via option
        if hasattr(options, "sequence"):
            if os.path.exists(options.sequence):  # Read file into content
                params[u'sequence'] = readFile(options.sequence)
            else:  # Argument is a sequence id
                params[u'sequence'] = options.sequence
        elif hasattr(options, "asequence") and hasattr(options, "bsequence"):
            if os.path.exists(options.asequence) and os.path.exists(options.bsequence):  # Read file into content
                params[u'asequence'] = readFile(options.asequence)
                params[u'bsequence'] = readFile(options.bsequence)
            else:  # Argument is a sequence id
                params[u'asequence'] = options.asequence
                params[u'bsequence'] = options.bsequence

    # Pass default values and fix bools (without default value)
    if options.database:
        params['database'] = options.database

    if not options.matrix:
        params['matrix'] = 'BLOSUM62'
    if options.matrix:
        params['matrix'] = options.matrix
    

    if not options.gapopen:
        params['gapopen'] = 11
    if options.gapopen:
        params['gapopen'] = options.gapopen
    

    if not options.gapext:
        params['gapext'] = 1
    if options.gapext:
        params['gapext'] = options.gapext
    

    if not options.expthr:
        params['expthr'] = '10.0'
    if options.expthr:
        params['expthr'] = options.expthr
    

    if not options.psithr:
        params['psithr'] = '1.0e-3'
    if options.psithr:
        params['psithr'] = options.psithr
    

    if not options.scores:
        params['scores'] = 500
    if options.scores:
        params['scores'] = options.scores
    

    if not options.alignments:
        params['alignments'] = 500
    if options.alignments:
        params['alignments'] = options.alignments
    

    if not options.alignView:
        params['alignView'] = 0
    if options.alignView:
        params['alignView'] = options.alignView
    

    if not options.dropoff:
        params['dropoff'] = 15
    if options.dropoff:
        params['dropoff'] = options.dropoff
    

    if not options.finaldropoff:
        params['finaldropoff'] = 25
    if options.finaldropoff:
        params['finaldropoff'] = options.finaldropoff
    

    if not options.filter:
        params['filter'] = 'F'
    if options.filter:
        params['filter'] = options.filter
    

    if options.seqrange:
        params['seqrange'] = options.seqrange
    

    if options.previousjobid:
        params['previousjobid'] = options.previousjobid
    

    if options.selectedHits:
        params['selectedHits'] = options.selectedHits
    

    if options.cpfile:
        params['cpfile'] = options.cpfile
    

    if not options.umode:
        params['umode'] = 'blastpgp'
    if options.umode:
        params['umode'] = options.umode
    

    if options.patfile:
        params['patfile'] = options.patfile
    


    # Submit the job
    if options.multifasta:
        multipleServiceRun(options.email, options.title, params,
                           options.useSeqId, options.maxJobs,
                           outputLevel)
    else:
        if options.useSeqId:
            print("Warning: --useSeqId option ignored.")
        if options.maxJobs:
            print("Warning: --maxJobs option ignored.")

        jobId = serviceRun(options.email, options.title, params)
        if options.asyncjob: # Async mode
            print(jobId)
            if outputLevel > 0:
                print("To check status: python %s --status --jobid %s"
                      "" % (os.path.basename(__file__), jobId))
        else:
            # Sync mode
            if outputLevel > 0:
                print("JobId: " + jobId, file=sys.stderr)
            else:
                print(jobId)
            time.sleep(pollFreq)
            getResult(jobId)

# Get job status
elif options.jobid and options.status:
    printGetStatus(options.jobid)

elif options.jobid and (options.resultTypes or options.polljob):
    status = serviceGetStatus(options.jobid)
    if status == 'PENDING' or status == 'RUNNING':
        print("Error: Job status is %s. "
              "To get result types the job must be finished." % status)
        quit()
    # List result types for job
    if options.resultTypes:
        printGetResultTypes(options.jobid)
    # Get results for job
    elif options.polljob:
        getResult(options.jobid)
else:
    # Checks for 'email' parameter
    if not options.email:
        print('\nParameter "--email" is missing in your command. It is required!\n')

    print(u'Error: unrecognised argument combination', file=sys.stderr)
    print_usage()



import core._ui_base as gui
__name__ = 'ncbiBlast_ui' 
import xml.etree.ElementTree as ET

# 'Global' variables used later
gui.service_cmd ="python3 ../embl_client/ncbiblast.py"
blast_hints = { 'blastp': 'Mathces a protein query to a protein database.',
                'blastn': 'Matches a nucleotide query to a nucleotide query',
                'blastx':  'Compares a DNA query to a protein database, by translating the query sequence in the 6 possible frames,\n' 
                           +'and comparing each against the database (3 reading frames from each strand of the DNA) searching.',
                'tblastn': 'Compares a protein query to a DNA database, in the 6 possible frames of the database.',
                'tblastx': 'Compares the protein encoded in a DNA query to the protein encoded in a DNA database,\n'
                           +'in the 6*6 possible frames of both query and database sequences (Note that all the combinations of frames may have different scores).' }
database_options = []
database_hints = {}
_file_path = gui.os.path.dirname(__file__)
databases_xml = ET.parse(_file_path+'/../core/ncbiBlast_databases.xml')
values = databases_xml.getroot().find('values');
for db in values:
    database_options.append((db.find('value').text, db.find('value').text))
    database_hints[db.find('value').text]=db.find('label').text 
    
#Defining UI elements / widgets
gui.app_label.value = 'EMBL-Tools NCBI BLAST+ webservice'

# Widgets for mandatory information, starting with predefined label and email input
    
#add more widgets as you need below this line
BLAST_program = gui.widgets.Dropdown(
    options = [('blastp','blastp'), ('blastn', 'blastn'),
               ('blastx', 'blastx'),('tblastx', 'tblastx'),
               ('tblastn', 'tblastn')],
    value='blastp',
    description='BLAST program:',
    style = gui.style
)

BLAST_program_hint = gui.widgets.HTML(value= '<style>p{word-wrap: break-word}</style> <p>BLAST program description: '+blast_hints[BLAST_program.value]+' </p>')

sequence_type = gui.widgets.Dropdown(
    options = [('DNA','dna'), ('RNA', 'rna'),('Protein', 'protein')],
    value='dna',
    description='Sequence type:',
    style = gui.style
)

database_dropdown= gui.widgets.Dropdown(
    options = database_options,
    value = database_options[0][1],
    description='Database to use:',
    style = gui.style
)

database_hint = gui.widgets.HTML(value= '<style>p{word-wrap: break-word}</style> <p>Database description: '+database_hints[database_dropdown.value]+' </p>')

def run_checks():
    if not gui.check_email():
        return False
    if not gui.check_file():
        return False
    return True
gui.run_checks = run_checks


# modify as needed.
def prepare_command():
    command = gui.service_cmd + ' --email ' + gui.email_input.value + ' --program '+ BLAST_program.value  + ' --stype ' + sequence_type.value + ' --sequence ' + gui.seq_file_input.selected + ' --database ' + database_dropdown.value 
    command += ' --asyncjob'
    return command
gui.prepare_command = prepare_command


def blast_hint(change):
    BLAST_program_hint.value = value= '<style>p{word-wrap: break-word}</style> <p>BLAST program description: '+blast_hints[change['new']]+'</p>'
    
BLAST_program.observe(blast_hint, names='value')

def database_hint_change(change):
    database_hint.value = value= '<style>p{word-wrap: break-word}</style> <p>Database description: '+database_hints[change['new']]+'</p>'
    
database_dropdown.observe(database_hint_change, names='value')


mandatory_options =[]
for widget in gui.mandatory_options.children:
    mandatory_options.append(widget)

mandatory_options.insert(2, sequence_type)
mandatory_options.insert(2, database_hint)
mandatory_options.insert(2, database_dropdown)
mandatory_options.insert(2, BLAST_program_hint)
mandatory_options.insert(2, BLAST_program)
optional_options =[]
for widget in gui.optional_options.children:
    optional_options.append(widget)

# Append or insert widgets to the optional widgets list with append() or insert()    
gui.mandatory_options.children = (mandatory_options)
gui.optional_options.children = (optional_options)
display(gui.app_layout)
display(gui.output)



import core._ui_base as gui
__name__ = 'emboss_backtranambig'
import xml.etree.ElementTree as ET
gui.service_cmd = 'python3 ../embl_client/emboss_backtranambig.py' 
gui.app_label.value= 'EMBL-Tools Backtranambig - Back-translate a protein sequence to ambiguous nucleotide sequence.Back-translate a protein sequence to ambiguous nucleotide sequence.'
codon_table_options = []
_file_path = gui.os.path.dirname(__file__)
databases_xml = ET.parse(_file_path+'/../core/Backtranambig_codon_table.xml')
values = databases_xml.getroot().find('values');
for option in values:
    codon_table_options.append((option.find('label').text, option.find('value').text))

# Define more widgets and add them to gui mandatory_options or gui.optional_options
# Add @gui.output.capture() to methods you want to have their output printed.
# The submit button is the last widget in the mandatory_options, thus you should instert widgets before that.
# It is recommended to insert any new widgets after the email field.
codon_table_dropdown= gui.widgets.Dropdown(
    options = codon_table_options,
    value = codon_table_options[0][1],
    description='Cotdon table to use:',
    style = gui.style,
    layout=gui.default_widget_layout
)

sequence_input_textarea = gui.widgets.Textarea(
    value='',
    placeholder='Sequence(s) to translate. Valid formats: GCG, FASTA, PIR, NBRF, PHYLIP or UniProtKB/Swiss-Prot.',
    description='Direct input for sequence(s):',
    disabled=False,
    rows=10,
    layout=gui.default_widget_layout,
    style = gui.style
)


@gui.output.capture()
def check_sequence_input():
    if ''.__eq__(sequence_input_textarea.value):
        print('Direct sequence input is empty.')
        return False
    return True

# Append more checks as needed. Define your check functions in the file and call them here.
# The method MUST only return true if all checks pass.
# use @gui.output.capture() annotation for the check method to capture it's output.
input_file_exists=False
direct_input=False
def run_checks():
    if not gui.check_email():
        return False
    input_file_exists =  gui.check_file()
    direct_input = check_sequence_input()
    if not input_file_exists and not direct_input:
        print('Either paste sequence into direct input or select a file.')
        return False
    if input_file_exists and direct_input:
        print('File upload and direct cannot be used at the same time. Please use only one of the input methods.')
        return False
    return True
gui.run_checks = run_checks

def write_tmp_seq_file():
    with open('/tmp/temp_seq_file', mode='w') as tempfile:
        tempfile.write(sequence_input_textarea.value)
        tempfile.close()
        
# Modify this method as needed to prepare the command to be executed
def prepare_command():
    sequence = None
    if input_file_exists:
        sequence = gui.seq_file_input.selected
    else:
        write_tmp_seq_file()
        sequence = '/tmp/temp_seq_file'
    command = f'''{gui.service_cmd} --email {gui.email_input.value} --sequence {sequence} --codontable {codon_table_dropdown.value}'''# add more options as needed for the base command    
    command += ' --asyncjob'
    return command
gui.prepare_command = prepare_command

mandatory_options =[]
for widget in gui.mandatory_options.children:
    mandatory_options.append(widget)

# Use the code below to insert widget just after email field.\
mandatory_options.insert(3, sequence_input_textarea)

optional_options =[]
for widget in gui.optional_options.children:
    optional_options.append(widget)
optional_options.insert(1, codon_table_dropdown)
# Append or insert widgets to the optional widgets list with append() or insert()    
gui.mandatory_options.children = (mandatory_options)
gui.optional_options.children = (optional_options)
display(gui.app_layout)
display(gui.output)